@@ -1,5 +1,5 @@
 #
-# Copyright 2013-2019 Ghent University
+# Copyright 2013-2020 Ghent University
 #
 # This file is part of EasyBuild,
 # originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
@@ -31,12 +31,14 @@ be used within an Easyconfig file.
 :author: Fotis Georgatos (Uni.Lu, NTUA)
 :author: Kenneth Hoste (Ghent University)
 """
+import copy
 import re
 import platform
-from vsc.utils import fancylogger
 
+from easybuild.base import fancylogger
 from easybuild.tools.build_log import EasyBuildError
-from easybuild.tools.systemtools import get_shared_lib_ext
+from easybuild.tools.py2vs3 import string_type
+from easybuild.tools.systemtools import get_shared_lib_ext, pick_dep_version
 
 
 _log = fancylogger.getLogger('easyconfig.templates', fname=False)
@@ -56,6 +58,7 @@ TEMPLATE_NAMES_CONFIG = [
     'bitbucket_account',
     'github_account',
     'name',
+    'parallel',
     'version',
     'versionsuffix',
     'versionprefix',
@@ -142,10 +145,13 @@ for ext in extensions:
 # versionmajor, versionminor, versionmajorminor (eg '.'.join(version.split('.')[:2])) )
 
 
-def template_constant_dict(config, ignore=None, skip_lower=True):
+def template_constant_dict(config, ignore=None, skip_lower=None):
     """Create a dict for templating the values in the easyconfigs.
         - config is a dict with the structure of EasyConfig._config
     """
+    if skip_lower is not None:
+        _log.deprecated("Use of 'skip_lower' named argument for template_constant_dict has no effect anymore", '4.0')
+
     # TODO find better name
     # ignore
     if ignore is None:
@@ -205,7 +211,15 @@ def template_constant_dict(config, ignore=None, skip_lower=True):
 
     # step 2: define *ver and *shortver templates
     for name, pref in TEMPLATE_SOFTWARE_VERSIONS:
-        for dep in config.get('dependencies', []):
+
+        # copy to avoid changing original list below
+        deps = copy.copy(config.get('dependencies', []))
+
+        # only consider build dependencies for defining *ver and *shortver templates if we're in iterative mode
+        if hasattr(config, 'iterating') and config.iterating:
+            deps += config.get('builddependencies', [])
+
+        for dep in deps:
             if isinstance(dep, dict):
                 dep_name, dep_version = dep['name'], dep['version']
             elif isinstance(dep, (list, tuple)):
@@ -213,9 +227,12 @@ def template_constant_dict(config, ignore=None, skip_lower=True):
             else:
                 raise EasyBuildError("Unexpected type for dependency: %s", dep)
 
-            if isinstance(dep_name, basestring) and dep_name.lower() == name.lower():
+            if isinstance(dep_name, string_type) and dep_name.lower() == name.lower():
+                dep_version = pick_dep_version(dep_version)
                 template_values['%sver' % pref] = dep_version
-                template_values['%sshortver' % pref] = '.'.join(dep_version.split('.')[:2])
+                dep_version_parts = dep_version.split('.')
+                template_values['%smajver' % pref] = dep_version_parts[0]
+                template_values['%sshortver' % pref] = '.'.join(dep_version_parts[:2])
                 break
 
     # step 3: add remaining from config
@@ -226,29 +243,31 @@ def template_constant_dict(config, ignore=None, skip_lower=True):
             template_values[name] = config[name]
             _log.debug('name: %s, config: %s', name, config[name])
 
-    # step 4. make lower variants if not skip_lower
-    if not skip_lower:
-        for name in TEMPLATE_NAMES_LOWER:
-            if name in ignore:
-                continue
-            t_v = template_values.get(name, None)
-            if t_v is None:
-                continue
-            try:
-                template_values[TEMPLATE_NAMES_LOWER_TEMPLATE % {'name': name}] = t_v.lower()
-            except Exception:
-                _log.debug("_getitem_string: can't get .lower() for name %s value %s (type %s)" %
-                           (name, t_v, type(t_v)))
+    # step 4. make lower variants
+    for name in TEMPLATE_NAMES_LOWER:
+        if name in ignore:
+            continue
+
+        value = config.get(name) or template_values.get(name)
+
+        if value is None:
+            continue
+        try:
+            template_values[TEMPLATE_NAMES_LOWER_TEMPLATE % {'name': name}] = value.lower()
+        except Exception:
+            _log.warning("Failed to get .lower() for name %s value %s (type %s)", name, value, type(value))
 
     return template_values
 
 
-def to_template_str(value, templ_const, templ_val):
+def to_template_str(key, value, templ_const, templ_val):
     """
     Insert template values where possible
-        - value is a string
-        - templ_const is a dictionary of template strings (constants)
-        - templ_val is an ordered dictionary of template strings specific for this easyconfig file
+
+    :param key: name of easyconfig parameter
+    :param value: string representing easyconfig parameter value
+    :param templ_const: dictionary of template strings (constants)
+    :param templ_val: (ordered) dictionary of template strings specific for this easyconfig file
     """
     old_value = None
     while value != old_value:
@@ -261,10 +280,16 @@ def to_template_str(value, templ_const, templ_val):
         for tval, tname in templ_val.items():
             # only replace full words with templates: word to replace should be at the beginning of a line
             # or be preceded by a non-alphanumeric (\W). It should end at the end of a line or be succeeded
-            # by another non-alphanumeric.
-            if tval in value:
+            # by another non-alphanumeric;
+            # avoid introducing self-referencing easyconfig parameter value
+            # by taking into account given name of easyconfig parameter ('key')
+            if tval in value and tname != key:
                 value = re.sub(r'(^|\W)' + re.escape(tval) + r'(\W|$)', r'\1%(' + tname + r')s\2', value)
 
+            # special case of %(pyshortver)s, where we should template 'python2.7' to 'python%(pyshortver)s'
+            if tname == 'pyshortver' and ('python' + tval) in value:
+                value = re.sub(r'(^|\W)python' + re.escape(tval) + r'(\W|$)', r'\1python%(' + tname + r')s\2', value)
+
     return value
 
 
