@@ -1,5 +1,5 @@
 ##
-# Copyright 2009-2019 Ghent University
+# Copyright 2009-2020 Ghent University
 #
 # This file is part of EasyBuild,
 # originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
@@ -41,9 +41,9 @@ import os
 import re
 import tempfile
 from distutils.version import LooseVersion
-from vsc.utils import fancylogger
-from vsc.utils.missing import nub
 
+from easybuild.base import fancylogger
+from easybuild.framework.easyconfig.constants import EASYCONFIG_CONSTANTS
 from easybuild.framework.easyconfig.default import get_easyconfig_parameter_default
 from easybuild.framework.easyconfig.easyconfig import EasyConfig, create_paths, process_easyconfig
 from easybuild.framework.easyconfig.easyconfig import get_toolchain_hierarchy, ActiveMNS
@@ -54,9 +54,9 @@ from easybuild.tools.config import build_option
 from easybuild.tools.filetools import read_file, write_file
 from easybuild.tools.module_naming_scheme.utilities import det_full_ec_version
 from easybuild.tools.robot import resolve_dependencies, robot_find_easyconfig
-from easybuild.tools.toolchain import DUMMY_TOOLCHAIN_NAME
+from easybuild.tools.toolchain.toolchain import SYSTEM_TOOLCHAIN_NAME
 from easybuild.tools.toolchain.toolchain import TOOLCHAIN_CAPABILITIES
-from easybuild.tools.utilities import quote_str
+from easybuild.tools.utilities import flatten, nub, quote_str
 
 
 _log = fancylogger.getLogger('easyconfig.tweak', fname=False)
@@ -100,7 +100,8 @@ def tweak(easyconfigs, build_specs, modtool, targetdirs=None):
         # Make sure there are no more build_specs, as combining --try-toolchain* with other options is currently not
         # supported
         if any(key not in ['toolchain_name', 'toolchain_version', 'toolchain'] for key in keys):
-            print_warning("Combining --try-toolchain* with other build options is not fully supported: using regex")
+            warning_msg = "Combining --try-toolchain* with other build options is not fully supported: using regex"
+            print_warning(warning_msg, silent=build_option('silent'))
             revert_to_regex = True
 
         if not revert_to_regex:
@@ -248,7 +249,13 @@ def tweak_one(orig_ec, tweaked_ec, tweaks, targetdir=None):
         if not res:
             raise EasyBuildError("No toolchain found in easyconfig file %s: %s", orig_ec, ectxt)
 
-        toolchain = eval(res.group(1))
+        # need to treat toolchain specified via 'SYSTEM' constant separately,
+        # since SYSTEM constant is not defined during 'eval'
+        if res.group(1) == 'SYSTEM':
+            toolchain = copy.copy(EASYCONFIG_CONSTANTS['SYSTEM'][0])
+        else:
+            toolchain = eval(res.group(1))
+
         for key in ['name', 'version']:
             tc_key = "toolchain_%s" % key
             if tc_key in keys:
@@ -271,7 +278,8 @@ def tweak_one(orig_ec, tweaked_ec, tweaks, targetdir=None):
         _log.warning("Tweaking version: checksums cleared, verification disabled.")
 
     # we need to treat list values separately, i.e. we prepend to the current value (if any)
-    for (key, val) in tweaks.items():
+    for key in list(tweaks):
+        val = tweaks[key]
 
         if isinstance(val, list):
             # use non-greedy matching for list value using '*?' to avoid including other parameters in match,
@@ -389,12 +397,12 @@ def pick_version(req_ver, avail_vers):
         if len(avail_vers) == 1:
             selected_ver = avail_vers[0]
         else:
-            retained_vers = [v for v in avail_vers if v <= LooseVersion(ver)]
+            retained_vers = [v for v in avail_vers if LooseVersion(v) <= LooseVersion(ver)]
             if retained_vers:
-                selected_ver = retained_vers[-1]
+                selected_ver = sorted(retained_vers, key=LooseVersion)[-1]
             else:
                 # if no versions are available that are less recent, take the least recent version
-                selected_ver = sorted([LooseVersion(v) for v in avail_vers])[0]
+                selected_ver = sorted(avail_vers, key=LooseVersion)[0]
     else:
         # if no desired version is specified, just use last version
         ver = avail_vers[-1]
@@ -454,7 +462,7 @@ def select_or_generate_ec(fp, paths, specs):
     # find ALL available easyconfig files for specified software
     cfg = {
         'version': '*',
-        'toolchain': {'name': DUMMY_TOOLCHAIN_NAME, 'version': '*'},
+        'toolchain': {'name': SYSTEM_TOOLCHAIN_NAME, 'version': '*'},
         'versionprefix': '*',
         'versionsuffix': '*',
     }
@@ -483,9 +491,12 @@ def select_or_generate_ec(fp, paths, specs):
     # TOOLCHAIN NAME
 
     # we can't rely on set, because we also need to be able to obtain a list of unique lists
-    def unique(lst):
+    def unique(lst, sortkey=None):
         """Retain unique elements in a sorted list."""
-        lst = sorted(lst)
+        if sortkey:
+            lst = sorted(lst, key=sortkey)
+        else:
+            lst = sorted(lst)
         if len(lst) > 1:
             res = [lst[0]]
             for x in lst:
@@ -535,8 +546,8 @@ def select_or_generate_ec(fp, paths, specs):
     _log.debug("Filtered easyconfigs: %s" % [x[1] for x in ecs_and_files])
 
     # TOOLCHAIN VERSION
-
-    tcvers = unique([x[0]['toolchain']['version'] for x in ecs_and_files])
+    tcvers = unique([x[0]['toolchain']['version'] for x in ecs_and_files if x[0]['toolchain']['version']],
+                    sortkey=LooseVersion)
     _log.debug("Found %d unique toolchain versions: %s" % (len(tcvers), tcvers))
 
     tcver = specs.pop('toolchain_version', None)
@@ -559,7 +570,8 @@ def select_or_generate_ec(fp, paths, specs):
 
     # SOFTWARE VERSION
 
-    vers = unique([x[0]['version'] for x in ecs_and_files])
+    vers = unique([x[0]['version'] for x in ecs_and_files if x[0]['version']], sortkey=LooseVersion)
+
     _log.debug("Found %d unique software versions: %s" % (len(vers), vers))
 
     ver = specs.pop('version', None)
@@ -743,7 +755,7 @@ def match_minimum_tc_specs(source_tc_spec, target_tc_hierarchy):
     # Warn if we are changing compiler families, this is very likely to cause problems
     if target_compiler_family != source_tc_spec['comp_family']:
         print_warning("Your request will result in a compiler family switch (%s to %s). Here be dragons!" %
-                      (source_tc_spec['comp_family'], target_compiler_family))
+                      (source_tc_spec['comp_family'], target_compiler_family), silent=build_option('silent'))
 
     return minimal_matching_toolchain
 
@@ -820,21 +832,29 @@ def map_easyconfig_to_target_tc_hierarchy(ec_spec, toolchain_mapping, targetdir=
     :return: Location of the modified easyconfig file
     """
     # Fully parse the original easyconfig
-    parsed_ec = process_easyconfig(ec_spec, validate=False)[0]
+    parsed_ec = process_easyconfig(ec_spec, validate=False)[0]['ec']
+
     # Replace the toolchain if the mapping exists
-    tc_name = parsed_ec['ec']['toolchain']['name']
+    tc_name = parsed_ec['toolchain']['name']
     if tc_name in toolchain_mapping:
         new_toolchain = toolchain_mapping[tc_name]
-        _log.debug("Replacing parent toolchain %s with %s", parsed_ec['ec']['toolchain'], new_toolchain)
-        parsed_ec['ec']['toolchain'] = new_toolchain
+        _log.debug("Replacing parent toolchain %s with %s", parsed_ec['toolchain'], new_toolchain)
+        parsed_ec['toolchain'] = new_toolchain
 
     # Replace the toolchains of all the dependencies
     for key in DEPENDENCY_PARAMETERS:
         # loop over a *copy* of dependency dicts (with resolved templates);
-        # to update the original dep dict, we need to index with idx into self._config[key][0]...
-        for idx, dep in enumerate(parsed_ec['ec'][key]):
+        # to update the original dep dict, we need to get a reference with templating disabled...
+        val = parsed_ec[key]
+        orig_val = parsed_ec.get_ref(key)
+
+        if key in parsed_ec.iterate_options:
+            val = flatten(val)
+            orig_val = flatten(orig_val)
+
+        for idx, dep in enumerate(val):
             # reference to original dep dict, this is the one we should be updating
-            orig_dep = parsed_ec['ec']._config[key][0][idx]
+            orig_dep = orig_val[idx]
             # skip dependencies that are marked as external modules
             if dep['external_module']:
                 continue
@@ -848,10 +868,10 @@ def map_easyconfig_to_target_tc_hierarchy(ec_spec, toolchain_mapping, targetdir=
                 orig_dep['short_mod_name'] = ActiveMNS().det_short_module_name(dep)
                 orig_dep['full_mod_name'] = ActiveMNS().det_full_module_name(dep)
     # Determine the name of the modified easyconfig and dump it to target_dir
-    ec_filename = '%s-%s.eb' % (parsed_ec['ec']['name'], det_full_ec_version(parsed_ec['ec']))
+    ec_filename = '%s-%s.eb' % (parsed_ec['name'], det_full_ec_version(parsed_ec))
     tweaked_spec = os.path.join(targetdir or tempfile.gettempdir(), ec_filename)
 
-    parsed_ec['ec'].dump(tweaked_spec, always_overwrite=False, backup=True)
+    parsed_ec.dump(tweaked_spec, always_overwrite=False, backup=True)
     _log.debug("Dumped easyconfig tweaked via --try-toolchain* to %s", tweaked_spec)
 
     return tweaked_spec
