@@ -1,5 +1,5 @@
 ##
-# Copyright 2015-2019 Ghent University
+# Copyright 2015-2020 Ghent University
 #
 # This file is part of EasyBuild,
 # originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
@@ -30,8 +30,8 @@ Abstract implementation of a package naming scheme.
 :author: Kenneth Hoste (Ghent University)
 """
 from abc import ABCMeta, abstractmethod
-from vsc.utils import fancylogger
 
+from easybuild.base import fancylogger
 from easybuild.tools.config import build_option
 
 
