@@ -1,5 +1,5 @@
 ##
-# Copyright 2011-2019 Ghent University
+# Copyright 2011-2020 Ghent University
 #
 # This file is part of EasyBuild,
 # originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
@@ -28,6 +28,7 @@ Module with useful functions for getting system information
 :author: Jens Timmerman (Ghent University)
 @auther: Ward Poelmans (Ghent University)
 """
+import ctypes
 import fcntl
 import grp  # @UnresolvedImport
 import os
@@ -37,17 +38,28 @@ import re
 import struct
 import sys
 import termios
+from ctypes.util import find_library
 from socket import gethostname
-from vsc.utils import fancylogger
-from vsc.utils.affinity import sched_getaffinity
 
+from easybuild.base import fancylogger
 from easybuild.tools.build_log import EasyBuildError
+from easybuild.tools.config import build_option
 from easybuild.tools.filetools import is_readable, read_file, which
+from easybuild.tools.py2vs3 import string_type
 from easybuild.tools.run import run_cmd
 
 
 _log = fancylogger.getLogger('systemtools', fname=False)
 
+
+try:
+    import distro
+    HAVE_DISTRO = True
+except ImportError as err:
+    _log.debug("Failed to import 'distro' Python module: %s", err)
+    HAVE_DISTRO = False
+
+
 # Architecture constants
 AARCH32 = 'AArch32'
 AARCH64 = 'AArch64'
@@ -102,6 +114,9 @@ VENDOR_IDS = {
     'AuthenticAMD': AMD,
     'GenuineIntel': INTEL,
     'IBM': IBM,
+    # IBM POWER9
+    '8335-GTH': IBM,
+    '8335-GTX': IBM,
 }
 # ARM Cortex part numbers from the corresponding ARM Processor Technical Reference Manuals,
 # see http://infocenter.arm.com - Cortex-A series processors, Section "Main ID Register"
@@ -127,6 +142,37 @@ class SystemToolsException(Exception):
     """raised when systemtools fails"""
 
 
+def sched_getaffinity():
+    """Determine list of available cores for current process."""
+    cpu_mask_t = ctypes.c_ulong
+    cpu_setsize = 1024
+    n_cpu_bits = 8 * ctypes.sizeof(cpu_mask_t)
+    n_mask_bits = cpu_setsize // n_cpu_bits
+
+    class cpu_set_t(ctypes.Structure):
+        """Class that implements the cpu_set_t struct."""
+        _fields_ = [('bits', cpu_mask_t * n_mask_bits)]
+
+    _libc_lib = find_library('c')
+    _libc = ctypes.cdll.LoadLibrary(_libc_lib)
+
+    pid = os.getpid()
+    cs = cpu_set_t()
+    ec = _libc.sched_getaffinity(os.getpid(), ctypes.sizeof(cpu_set_t), ctypes.pointer(cs))
+    if ec == 0:
+        _log.debug("sched_getaffinity for pid %s successful", pid)
+    else:
+        raise EasyBuildError("sched_getaffinity failed for pid %s ec %s", pid, ec)
+
+    cpus = []
+    for bitmask in cs.bits:
+        for _ in range(n_cpu_bits):
+            cpus.append(bitmask & 1)
+            bitmask >>= 1
+
+    return cpus
+
+
 def get_avail_core_count():
     """
     Returns the number of available CPUs, according to cgroups and taskssets limits
@@ -136,7 +182,7 @@ def get_avail_core_count():
 
     if os_type == LINUX:
         # simple use available sched_getaffinity() function (yields a long, so cast it down to int)
-        core_cnt = int(sum(sched_getaffinity().cpus))
+        core_cnt = int(sum(sched_getaffinity()))
     else:
         # BSD-type systems
         out, _ = run_cmd('sysctl -n hw.ncpu', force_in_dry_run=True, trace=False, stream_output=False)
@@ -171,14 +217,14 @@ def get_total_memory():
         meminfo = read_file(PROC_MEMINFO_FP)
         mem_mo = re.match(r'^MemTotal:\s*(\d+)\s*kB', meminfo, re.M)
         if mem_mo:
-            memtotal = int(mem_mo.group(1)) / 1024
+            memtotal = int(mem_mo.group(1)) // 1024
 
     elif os_type == DARWIN:
         cmd = "sysctl -n hw.memsize"
         _log.debug("Trying to determine total memory size on Darwin via cmd '%s'", cmd)
         out, ec = run_cmd(cmd, force_in_dry_run=True, trace=False, stream_output=False)
         if ec == 0:
-            memtotal = int(out.strip()) / (1024**2)
+            memtotal = int(out.strip()) // (1024**2)
 
     if memtotal is None:
         memtotal = UNKNOWN
@@ -233,7 +279,7 @@ def get_cpu_vendor():
         if arch == X86_64:
             vendor_regex = re.compile(r"vendor_id\s+:\s*(\S+)")
         elif arch == POWER:
-            vendor_regex = re.compile(r"model\s+:\s*(\w+)")
+            vendor_regex = re.compile(r"model\s+:\s*((\w|-)+)")
         elif arch in [AARCH32, AARCH64]:
             vendor_regex = re.compile(r"CPU implementer\s+:\s*(\S+)")
 
@@ -361,7 +407,7 @@ def get_cpu_speed():
         if is_readable(MAX_FREQ_FP):
             _log.debug("Trying to determine CPU frequency on Linux via %s" % MAX_FREQ_FP)
             txt = read_file(MAX_FREQ_FP)
-            cpu_freq = float(txt) / 1000
+            cpu_freq = float(txt) // 1000
 
         # Linux without cpu scaling
         elif is_readable(PROC_CPUINFO_FP):
@@ -384,7 +430,7 @@ def get_cpu_speed():
         out, ec = run_cmd(cmd, force_in_dry_run=True, trace=False, stream_output=False)
         if ec == 0:
             # returns clock frequency in cycles/sec, but we want MHz
-            cpu_freq = float(out.strip()) / (1000 ** 2)
+            cpu_freq = float(out.strip()) // (1000 ** 2)
 
     else:
         raise SystemToolsException("Could not determine CPU clock frequency (OS: %s)." % os_type)
@@ -499,9 +545,21 @@ def get_os_name():
     Determine system name, e.g., 'redhat' (generic), 'centos', 'debian', 'fedora', 'suse', 'ubuntu',
     'red hat enterprise linux server', 'SL' (Scientific Linux), 'opensuse', ...
     """
-    # platform.linux_distribution is more useful, but only available since Python 2.6
-    # this allows to differentiate between Fedora, CentOS, RHEL and Scientific Linux (Rocks is just CentOS)
-    os_name = platform.linux_distribution()[0].strip().lower()
+    os_name = None
+
+    # platform.linux_distribution was removed in Python 3.8,
+    # see https://docs.python.org/2/library/platform.html#platform.linux_distribution
+    if hasattr(platform, 'linux_distribution'):
+        # platform.linux_distribution is more useful, but only available since Python 2.6
+        # this allows to differentiate between Fedora, CentOS, RHEL and Scientific Linux (Rocks is just CentOS)
+        os_name = platform.linux_distribution()[0].strip().lower()
+    elif HAVE_DISTRO:
+        # distro package is the recommended alternative to platform.linux_distribution,
+        # see https://pypi.org/project/distro
+        os_name = distro.name()
+    else:
+        # no easy way to determine name of Linux distribution
+        os_name = None
 
     os_name_map = {
         'red hat enterprise linux server': 'RHEL',
@@ -518,7 +576,15 @@ def get_os_name():
 
 def get_os_version():
     """Determine system version."""
-    os_version = platform.dist()[1]
+
+    # platform.dist was removed in Python 3.8
+    if hasattr(platform, 'dist'):
+        os_version = platform.dist()[1]
+    elif HAVE_DISTRO:
+        os_version = distro.version()
+    else:
+        os_version = None
+
     if os_version:
         if get_os_name() in ["suse", "SLES"]:
 
@@ -722,7 +788,7 @@ def use_group(group_name):
 def det_parallelism(par=None, maxpar=None):
     """
     Determine level of parallelism that should be used.
-    Default: educated guess based on # cores and 'ulimit -u' setting: min(# cores, ((ulimit -u) - 15) / 6)
+    Default: educated guess based on # cores and 'ulimit -u' setting: min(# cores, ((ulimit -u) - 15) // 6)
     """
     if par is not None:
         if not isinstance(par, int):
@@ -739,7 +805,7 @@ def det_parallelism(par=None, maxpar=None):
                 out = 2 ** 32 - 1
             maxuserproc = int(out)
             # assume 6 processes per build thread + 15 overhead
-            par_guess = int((maxuserproc - 15) / 6)
+            par_guess = int((maxuserproc - 15) // 6)
             if par_guess < par:
                 par = par_guess
                 _log.info("Limit parallel builds to %s because max user processes is %s" % (par, out))
@@ -770,3 +836,72 @@ def det_terminal_size():
             height, width = 25, 80
 
     return height, width
+
+
+def check_python_version():
+    """Check currently used Python version."""
+    python_maj_ver = sys.version_info[0]
+    python_min_ver = sys.version_info[1]
+    python_ver = '%d.%d' % (python_maj_ver, python_min_ver)
+    _log.info("Found Python version %s", python_ver)
+
+    silence_deprecation_warnings = build_option('silence_deprecation_warnings') or []
+
+    if python_maj_ver == 2:
+        if python_min_ver < 6:
+            raise EasyBuildError("Python 2.6 or higher is required when using Python 2, found Python %s", python_ver)
+        elif python_min_ver == 6:
+            depr_msg = "Running EasyBuild with Python 2.6 is deprecated"
+            if 'Python26' in silence_deprecation_warnings:
+                _log.warning(depr_msg)
+            else:
+                _log.deprecated(depr_msg, '5.0')
+        else:
+            _log.info("Running EasyBuild with Python 2 (version %s)", python_ver)
+
+    elif python_maj_ver == 3:
+        if python_min_ver < 5:
+            raise EasyBuildError("Python 3.5 or higher is required when using Python 3, found Python %s", python_ver)
+        else:
+            _log.info("Running EasyBuild with Python 3 (version %s)", python_ver)
+    else:
+        raise EasyBuildError("EasyBuild is not compatible (yet) with Python %s", python_ver)
+
+    return (python_maj_ver, python_min_ver)
+
+
+def pick_dep_version(dep_version):
+    """
+    Pick the correct dependency version to use for this system.
+    Input can either be:
+    * a string value (or None)
+    * a dict with options to choose from
+
+    Return value is the version to use.
+    """
+    if isinstance(dep_version, string_type):
+        _log.debug("Version is already a string ('%s'), OK", dep_version)
+        result = dep_version
+
+    elif dep_version is None:
+        _log.debug("Version is None, OK")
+        result = None
+
+    elif isinstance(dep_version, dict):
+        # figure out matches based on dict keys (after splitting on '=')
+        my_arch_key = 'arch=%s' % get_cpu_architecture()
+        arch_keys = [x for x in dep_version.keys() if x.startswith('arch=')]
+        other_keys = [x for x in dep_version.keys() if x not in arch_keys]
+        if other_keys:
+            raise EasyBuildError("Unexpected keys in version: %s. Only 'arch=' keys are supported", other_keys)
+        if arch_keys:
+            if my_arch_key in dep_version:
+                result = dep_version[my_arch_key]
+                _log.info("Version selected from %s using key %s: %s", dep_version, my_arch_key, result)
+            else:
+                raise EasyBuildError("No matches for version in %s (looking for %s)", dep_version, my_arch_key)
+
+    else:
+        raise EasyBuildError("Unknown value type for version: %s", dep_version)
+
+    return result
