@@ -1,5 +1,5 @@
 # #
-# Copyright 2015-2019 Ghent University
+# Copyright 2015-2020 Ghent University
 #
 # This file is part of EasyBuild,
 # originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
@@ -34,10 +34,11 @@ import tempfile
 from datetime import datetime, timedelta
 from test.framework.utilities import EnhancedTestCase, TestLoaderFiltered
 from unittest import TextTestRunner
-from vsc.utils.fancylogger import getLogger, getRootLoggerName, logToFile, setLogFormat
 
-from easybuild.tools.build_log import LOGGING_FORMAT, EasyBuildError, dry_run_msg, dry_run_warning
-from easybuild.tools.build_log import print_error, print_msg, print_warning, time_str_since
+from easybuild.base.fancylogger import getLogger, logToFile, setLogFormat
+from easybuild.tools.build_log import (
+    LOGGING_FORMAT, EasyBuildError, EasyBuildLog, dry_run_msg, dry_run_warning, init_logging, print_error, print_msg,
+    print_warning, stop_logging, time_str_since, raise_nosupport)
 from easybuild.tools.filetools import read_file, write_file
 
 
@@ -68,7 +69,7 @@ class BuildLogTest(EnhancedTestCase):
         self.assertErrorRegex(EasyBuildError, 'BOOM', raise_easybuilderror, 'BOOM')
         logToFile(tmplog, enable=False)
 
-        log_re = re.compile("^%s ::.* BOOM \(at .*:[0-9]+ in [a-z_]+\)$" % getRootLoggerName(), re.M)
+        log_re = re.compile(r"^root ::.* BOOM \(at .*:[0-9]+ in [a-z_]+\)$", re.M)
         logtxt = open(tmplog, 'r').read()
         self.assertTrue(log_re.match(logtxt), "%s matches %s" % (log_re.pattern, logtxt))
 
@@ -105,10 +106,11 @@ class BuildLogTest(EnhancedTestCase):
         log.setLevelName('DEBUG')
         log.debug("123 debug")
         log.info("foobar info")
-        log.warn("justawarning")
+        log.warning("justawarning")
         log.deprecated("anotherwarning", newer_ver)
         log.deprecated("onemorewarning", '1.0', '2.0')
         log.deprecated("lastwarning", '1.0', max_ver='2.0')
+        log.deprecated("thisisnotprinted", '1.0', max_ver='2.0', silent=True)
         log.error("kaput")
         log.error("err: %s", 'msg: %s')
         stderr = self.get_stderr()
@@ -129,33 +131,33 @@ class BuildLogTest(EnhancedTestCase):
         logToFile(tmplog, enable=False)
         logtxt = read_file(tmplog)
 
-        root = getRootLoggerName()
-
         expected_logtxt = '\n'.join([
-            r"%s.test_easybuildlog \[DEBUG\] :: 123 debug" % root,
-            r"%s.test_easybuildlog \[INFO\] :: foobar info" % root,
-            r"%s.test_easybuildlog \[WARNING\] :: justawarning" % root,
-            r"%s.test_easybuildlog \[WARNING\] :: Deprecated functionality.*anotherwarning.*" % root,
-            r"%s.test_easybuildlog \[WARNING\] :: Deprecated functionality.*onemorewarning.*" % root,
-            r"%s.test_easybuildlog \[WARNING\] :: Deprecated functionality.*lastwarning.*" % root,
-            r"%s.test_easybuildlog \[ERROR\] :: EasyBuild crashed with an error \(at .* in .*\): kaput" % root,
-            root + r".test_easybuildlog \[ERROR\] :: EasyBuild crashed with an error \(at .* in .*\): err: msg: %s",
-            r"%s.test_easybuildlog \[ERROR\] :: .*EasyBuild encountered an exception \(at .* in .*\): oops" % root,
+            r"root.test_easybuildlog \[DEBUG\] :: 123 debug",
+            r"root.test_easybuildlog \[INFO\] :: foobar info",
+            r"root.test_easybuildlog \[WARNING\] :: justawarning",
+            r"root.test_easybuildlog \[WARNING\] :: Deprecated functionality.*anotherwarning.*",
+            r"root.test_easybuildlog \[WARNING\] :: Deprecated functionality.*onemorewarning.*",
+            r"root.test_easybuildlog \[WARNING\] :: Deprecated functionality.*lastwarning.*",
+            r"root.test_easybuildlog \[WARNING\] :: Deprecated functionality.*thisisnotprinted.*",
+            r"root.test_easybuildlog \[ERROR\] :: EasyBuild crashed with an error \(at .* in .*\): kaput",
+            r"root.test_easybuildlog \[ERROR\] :: EasyBuild crashed with an error \(at .* in .*\): err: msg: %s",
+            r"root.test_easybuildlog \[ERROR\] :: .*EasyBuild encountered an exception \(at .* in .*\): oops",
             '',
         ])
         logtxt_regex = re.compile(r'^%s' % expected_logtxt, re.M)
         self.assertTrue(logtxt_regex.search(logtxt), "Pattern '%s' found in %s" % (logtxt_regex.pattern, logtxt))
 
-        self.assertErrorRegex(EasyBuildError, "DEPRECATED \(since .*: kaput", log.deprecated, "kaput", older_ver)
-        self.assertErrorRegex(EasyBuildError, "DEPRECATED \(since .*: 2>1", log.deprecated, "2>1", '2.0', '1.0')
-        self.assertErrorRegex(EasyBuildError, "DEPRECATED \(since .*: 2>1", log.deprecated, "2>1", '2.0', max_ver='1.0')
+        self.assertErrorRegex(EasyBuildError, r"DEPRECATED \(since .*: kaput", log.deprecated, "kaput", older_ver)
+        self.assertErrorRegex(EasyBuildError, r"DEPRECATED \(since .*: 2>1", log.deprecated, "2>1", '2.0', '1.0')
+        self.assertErrorRegex(EasyBuildError, r"DEPRECATED \(since .*: 2>1", log.deprecated, "2>1", '2.0',
+                              max_ver='1.0')
 
         # wipe log so we can reuse it
         write_file(tmplog, '')
 
         # test formatting log messages by providing extra arguments
         logToFile(tmplog, enable=True)
-        log.warn("%s", "bleh"),
+        log.warning("%s", "bleh")
         log.info("%s+%s = %d", '4', '2', 42)
         args = ['this', 'is', 'just', 'a', 'test']
         log.debug("%s %s %s %s %s", *args)
@@ -163,10 +165,10 @@ class BuildLogTest(EnhancedTestCase):
         logToFile(tmplog, enable=False)
         logtxt = read_file(tmplog)
         expected_logtxt = '\n'.join([
-            r"%s.test_easybuildlog \[WARNING\] :: bleh" % root,
-            r"%s.test_easybuildlog \[INFO\] :: 4\+2 = 42" % root,
-            r"%s.test_easybuildlog \[DEBUG\] :: this is just a test" % root,
-            r"%s.test_easybuildlog \[ERROR\] :: EasyBuild crashed with an error \(at .* in .*\): foo baz baz" % root,
+            r"root.test_easybuildlog \[WARNING\] :: bleh",
+            r"root.test_easybuildlog \[INFO\] :: 4\+2 = 42",
+            r"root.test_easybuildlog \[DEBUG\] :: this is just a test",
+            r"root.test_easybuildlog \[ERROR\] :: EasyBuild crashed with an error \(at .* in .*\): foo baz baz",
             '',
         ])
         logtxt_regex = re.compile(r'^%s' % expected_logtxt, re.M)
@@ -215,9 +217,7 @@ class BuildLogTest(EnhancedTestCase):
         logToFile(tmplog, enable=False)
         logtxt = read_file(tmplog)
 
-        root = getRootLoggerName()
-
-        prefix = '%s.test_easybuildlog' % root
+        prefix = 'root.test_easybuildlog'
         devel_msg = r"%s \[DEVEL\] :: tmi" % prefix
         debug_msg = r"%s \[DEBUG\] :: gdb" % prefix
         info_msg = r"%s \[INFO\] :: fyi" % prefix
@@ -370,6 +370,64 @@ class BuildLogTest(EnhancedTestCase):
 
         self.assertErrorRegex(EasyBuildError, "Unknown named arguments", dry_run_warning, 'foo', unknown_arg='bar')
 
+    def test_init_logging(self):
+        """Test init_logging function."""
+        # first, make very sure $TMPDIR is a subdir of self.test_prefix
+        tmpdir = os.getenv('TMPDIR')
+        self.assertTrue(tmpdir.startswith(self.test_prefix))
+
+        # use provided path for log file
+        tmp_logfile = os.path.join(self.test_prefix, 'test.log')
+        log, logfile = init_logging(tmp_logfile, silent=True)
+        self.assertEqual(logfile, tmp_logfile)
+        self.assertTrue(os.path.exists(logfile))
+        self.assertTrue(isinstance(log, EasyBuildLog))
+
+        stop_logging(logfile)
+
+        # no log provided, so create one (should be file in $TMPDIR)
+        log, logfile = init_logging(None, silent=True)
+        self.assertTrue(os.path.exists(logfile))
+        self.assertEqual(os.path.dirname(logfile), tmpdir)
+        self.assertTrue(isinstance(log, EasyBuildLog))
+
+        stop_logging(logfile)
+
+        # no problem with specifying a different directory to put log file in (even if it doesn't exist yet)
+        tmp_logdir = os.path.join(self.test_prefix, 'tmp_logs')
+        self.assertFalse(os.path.exists(tmp_logdir))
+
+        log, logfile = init_logging(None, silent=True, tmp_logdir=tmp_logdir)
+        self.assertEqual(os.path.dirname(logfile), tmp_logdir)
+        self.assertTrue(isinstance(log, EasyBuildLog))
+
+        stop_logging(logfile)
+
+        # by default, path to tmp log file is printed
+        self.mock_stdout(True)
+        log, logfile = init_logging(None)
+        stdout = self.get_stdout()
+        self.mock_stdout(False)
+        self.assertTrue(os.path.exists(logfile))
+        self.assertEqual(os.path.dirname(logfile), tmpdir)
+        self.assertTrue(isinstance(log, EasyBuildLog))
+        self.assertTrue(stdout.startswith("== temporary log file in case of crash"))
+
+        stop_logging(logfile)
+
+        # logging to stdout implies no log file
+        self.mock_stdout(True)
+        log, logfile = init_logging(None, logtostdout=True)
+        self.mock_stdout(False)
+        self.assertEqual(logfile, None)
+        self.assertTrue(isinstance(log, EasyBuildLog))
+
+        stop_logging(logfile, logtostdout=True)
+
+    def test_raise_nosupport(self):
+        self.assertErrorRegex(EasyBuildError, 'NO LONGER SUPPORTED since v42: foobar;',
+                              raise_nosupport, 'foobar', 42)
+
 
 def suite():
     """ returns all the testcases in this module """
@@ -377,4 +435,5 @@ def suite():
 
 
 if __name__ == '__main__':
-    TextTestRunner(verbosity=1).run(suite())
+    res = TextTestRunner(verbosity=1).run(suite())
+    sys.exit(len(res.failures))
