@@ -1,5 +1,5 @@
 ##
-# Copyright 2012-2019 Ghent University
+# Copyright 2012-2020 Ghent University
 #
 # This file is part of EasyBuild,
 # originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
@@ -33,6 +33,8 @@ import re
 import shutil
 import sys
 import tempfile
+from inspect import cleandoc
+from datetime import datetime
 from test.framework.utilities import EnhancedTestCase, TestLoaderFiltered, init_config
 from unittest import TextTestRunner
 
@@ -47,6 +49,7 @@ from easybuild.tools.config import get_module_syntax
 from easybuild.tools.filetools import copy_dir, copy_file, mkdir, read_file, remove_file, write_file
 from easybuild.tools.module_generator import module_generator
 from easybuild.tools.modules import reset_module_caches
+from easybuild.tools.utilities import time2str
 from easybuild.tools.version import get_git_revision, this_is_easybuild
 
 
@@ -97,7 +100,7 @@ class EasyBlockTest(EnhancedTestCase):
             'version = "%s"' % version,
             'homepage = "http://example.com"',
             'description = "test easyconfig"',
-            'toolchain = {"name":"dummy", "version": "dummy"}',
+            'toolchain = SYSTEM',
             'exts_list = ["ext1"]',
         ])
         self.writeEC()
@@ -214,7 +217,7 @@ class EasyBlockTest(EnhancedTestCase):
             'version = "3.14"',
             'homepage = "http://example.com"',
             'description = "test easyconfig"',
-            'toolchain = {"name": "dummy", "version": "dummy"}',
+            'toolchain = SYSTEM',
         ])
         self.writeEC()
         eb = EasyBlock(EasyConfig(self.eb_file))
@@ -249,7 +252,7 @@ class EasyBlockTest(EnhancedTestCase):
             'version = "3.14"',
             'homepage = "http://example.com"',
             'description = "test easyconfig"',
-            'toolchain = {"name":"dummy", "version": "dummy"}',
+            'toolchain = SYSTEM',
             'moduleclass = "compiler"',
         ])
         self.writeEC()
@@ -305,7 +308,7 @@ class EasyBlockTest(EnhancedTestCase):
             'version = "3.14"',
             'homepage = "http://example.com"',
             'description = "test easyconfig"',
-            'toolchain = {"name":"dummy", "version": "dummy"}',
+            'toolchain = SYSTEM',
         ])
         self.writeEC()
         eb = EasyBlock(EasyConfig(self.eb_file))
@@ -316,6 +319,8 @@ class EasyBlockTest(EnhancedTestCase):
         open(os.path.join(eb.installdir, 'foo.jar'), 'w').write('foo.jar')
         open(os.path.join(eb.installdir, 'bla.jar'), 'w').write('bla.jar')
         os.mkdir(os.path.join(eb.installdir, 'bin'))
+        os.mkdir(os.path.join(eb.installdir, 'bin', 'testdir'))
+        os.mkdir(os.path.join(eb.installdir, 'sbin'))
         os.mkdir(os.path.join(eb.installdir, 'share'))
         os.mkdir(os.path.join(eb.installdir, 'share', 'man'))
         # this is not a path that should be picked up
@@ -327,17 +332,35 @@ class EasyBlockTest(EnhancedTestCase):
             self.assertTrue(re.search(r"^prepend-path\s+CLASSPATH\s+\$root/bla.jar$", guess, re.M))
             self.assertTrue(re.search(r"^prepend-path\s+CLASSPATH\s+\$root/foo.jar$", guess, re.M))
             self.assertTrue(re.search(r"^prepend-path\s+MANPATH\s+\$root/share/man$", guess, re.M))
-            self.assertTrue(re.search(r"^prepend-path\s+PATH\s+\$root/bin$", guess, re.M))
+            # bin/ is not added to $PATH if it doesn't include files
+            self.assertFalse(re.search(r"^prepend-path\s+PATH\s+\$root/bin$", guess, re.M))
+            self.assertFalse(re.search(r"^prepend-path\s+PATH\s+\$root/sbin$", guess, re.M))
+            # no include/ subdirectory, so no $CPATH update statement
             self.assertFalse(re.search(r"^prepend-path\s+CPATH\s+.*$", guess, re.M))
         elif get_module_syntax() == 'Lua':
             self.assertTrue(re.search(r'^prepend_path\("CLASSPATH", pathJoin\(root, "bla.jar"\)\)$', guess, re.M))
             self.assertTrue(re.search(r'^prepend_path\("CLASSPATH", pathJoin\(root, "foo.jar"\)\)$', guess, re.M))
             self.assertTrue(re.search(r'^prepend_path\("MANPATH", pathJoin\(root, "share/man"\)\)$', guess, re.M))
-            self.assertTrue(re.search(r'^prepend_path\("PATH", pathJoin\(root, "bin"\)\)$', guess, re.M))
+            # bin/ is not added to $PATH if it doesn't include files
+            self.assertFalse(re.search(r'^prepend_path\("PATH", pathJoin\(root, "bin"\)\)$', guess, re.M))
+            self.assertFalse(re.search(r'^prepend_path\("PATH", pathJoin\(root, "sbin"\)\)$', guess, re.M))
+            # no include/ subdirectory, so no $CPATH update statement
             self.assertFalse(re.search(r'^prepend_path\("CPATH", .*\)$', guess, re.M))
         else:
             self.assertTrue(False, "Unknown module syntax: %s" % get_module_syntax())
 
+        # check that bin is only added to PATH if there are files in there
+        open(os.path.join(eb.installdir, 'bin', 'test'), 'w').write('test')
+        guess = eb.make_module_req()
+        if get_module_syntax() == 'Tcl':
+            self.assertTrue(re.search(r"^prepend-path\s+PATH\s+\$root/bin$", guess, re.M))
+            self.assertFalse(re.search(r"^prepend-path\s+PATH\s+\$root/sbin$", guess, re.M))
+        elif get_module_syntax() == 'Lua':
+            self.assertTrue(re.search(r'^prepend_path\("PATH", pathJoin\(root, "bin"\)\)$', guess, re.M))
+            self.assertFalse(re.search(r'^prepend_path\("PATH", pathJoin\(root, "sbin"\)\)$', guess, re.M))
+        else:
+            self.assertTrue(False, "Unknown module syntax: %s" % get_module_syntax())
+
         # check for behavior when a string value is used as dict value by make_module_req_guesses
         eb.make_module_req_guess = lambda: {'PATH': 'bin'}
         txt = eb.make_module_req()
@@ -367,6 +390,8 @@ class EasyBlockTest(EnhancedTestCase):
 
     def test_make_module_extra(self):
         """Test for make_module_extra."""
+        init_config(build_options={'silent': True})
+
         self.contents = '\n'.join([
             'easyblock = "ConfigureMake"',
             'name = "pi"',
@@ -456,6 +481,8 @@ class EasyBlockTest(EnhancedTestCase):
 
     def test_make_module_dep(self):
         """Test for make_module_dep"""
+        init_config(build_options={'silent': True})
+
         self.contents = '\n'.join([
             'easyblock = "ConfigureMake"',
             'name = "pi"',
@@ -494,17 +521,17 @@ class EasyBlockTest(EnhancedTestCase):
             ])
         elif get_module_syntax() == 'Lua':
             tc_load = '\n'.join([
-                'if not isloaded("gompi/2018a") then',
+                'if not ( isloaded("gompi/2018a") ) then',
                 '    load("gompi/2018a")',
                 'end',
             ])
             fftw_load = '\n'.join([
-                'if not isloaded("FFTW/3.3.7-gompi-2018a") then',
+                'if not ( isloaded("FFTW/3.3.7-gompi-2018a") ) then',
                 '    load("FFTW/3.3.7-gompi-2018a")',
                 'end',
             ])
             lapack_load = '\n'.join([
-                'if not isloaded("OpenBLAS/0.2.20-GCC-6.4.0-2.28") then',
+                'if not ( isloaded("OpenBLAS/0.2.20-GCC-6.4.0-2.28") ) then',
                 '    load("OpenBLAS/0.2.20-GCC-6.4.0-2.28")',
                 'end',
             ])
@@ -528,7 +555,7 @@ class EasyBlockTest(EnhancedTestCase):
             ])
         elif get_module_syntax() == 'Lua':
             fftw_load = '\n'.join([
-                'if not isloaded("FFTW/3.3.7-gompi-2018a") then',
+                'if not ( isloaded("FFTW/3.3.7-gompi-2018a") ) then',
                 '    unload("FFTW")',
                 '    load("FFTW/3.3.7-gompi-2018a")',
                 'end',
@@ -545,6 +572,7 @@ class EasyBlockTest(EnhancedTestCase):
         build_options = {
             'check_osdeps': False,
             'robot_path': [test_ecs_path],
+            'silent': True,
             'valid_stops': all_stops,
             'validate': False,
         }
@@ -623,15 +651,113 @@ class EasyBlockTest(EnhancedTestCase):
             regex = re.compile('load.*%s' % mod)
             self.assertFalse(regex.search(mod_dep_txt), "Pattern '%s' found in: %s" % (regex.pattern, mod_dep_txt))
 
-    def test_extensions_step(self):
-        """Test the extensions_step"""
+    def test_det_iter_cnt(self):
+        """Test det_iter_cnt method."""
+
         self.contents = '\n'.join([
             'easyblock = "ConfigureMake"',
             'name = "pi"',
             'version = "3.14"',
             'homepage = "http://example.com"',
             'description = "test easyconfig"',
-            'toolchain = {"name": "dummy", "version": "dummy"}',
+            'toolchain = SYSTEM',
+        ])
+
+        self.writeEC()
+        eb = EasyBlock(EasyConfig(self.eb_file))
+
+        # default value should be 1
+        self.assertEqual(eb.det_iter_cnt(), 1)
+
+        # adding a list of build deps shouldn't affect the default
+        self.contents += "\nbuilddependencies = [('one', '1.0'), ('two', '2.0'), ('three', '3.0')]"
+        self.writeEC()
+        eb = EasyBlock(EasyConfig(self.eb_file))
+        self.assertEqual(eb.det_iter_cnt(), 1)
+
+        # list of configure options to iterate over affects iteration count
+        self.contents += "\nconfigopts = ['--one', '--two', '--three', '--four']"
+        self.writeEC()
+        eb = EasyBlock(EasyConfig(self.eb_file))
+        self.assertEqual(eb.det_iter_cnt(), 4)
+
+        # different lengths for iterative easyconfig parameters mean trouble during validation of iterative parameters
+        self.contents += "\nbuildopts = ['FOO=one', 'FOO=two']"
+        self.writeEC()
+
+        error_pattern = "lists for iterated build should have same length"
+        self.assertErrorRegex(EasyBuildError, error_pattern, EasyConfig, self.eb_file)
+
+    def test_handle_iterate_opts(self):
+        """Test for handle_iterate_opts method."""
+        testdir = os.path.abspath(os.path.dirname(__file__))
+        toy_ec = os.path.join(testdir, 'easyconfigs', 'test_ecs', 't', 'toy', 'toy-0.0.eb')
+
+        test_ec = os.path.join(self.test_prefix, 'test.eb')
+        write_file(test_ec, read_file(toy_ec) + "\nconfigopts = ['--opt1 --anotheropt', '--opt2', '--opt3 --optbis']")
+
+        ec = process_easyconfig(test_ec)[0]
+        eb = get_easyblock_instance(ec)
+
+        # check initial state
+        self.assertEqual(eb.iter_idx, 0)
+        self.assertEqual(eb.iter_opts, {})
+        self.assertEqual(eb.cfg.iterating, False)
+        self.assertEqual(eb.cfg.iterate_options, [])
+        self.assertEqual(eb.cfg['configopts'], ["--opt1 --anotheropt", "--opt2", "--opt3 --optbis"])
+
+        expected_iter_opts = {'configopts': ["--opt1 --anotheropt", "--opt2", "--opt3 --optbis"]}
+
+        # once iteration mode is set, we're still in iteration #0
+        self.mock_stdout(True)
+        eb.handle_iterate_opts()
+        stdout = self.get_stdout()
+        self.mock_stdout(False)
+        self.assertEqual(eb.iter_idx, 0)
+        self.assertEqual(stdout, "== starting iteration #0 ...\n")
+        self.assertEqual(eb.cfg.iterating, True)
+        self.assertEqual(eb.cfg.iterate_options, ['configopts'])
+        self.assertEqual(eb.cfg['configopts'], "--opt1 --anotheropt")
+        self.assertEqual(eb.iter_opts, expected_iter_opts)
+
+        # when next iteration is start, iteration index gets bumped
+        self.mock_stdout(True)
+        eb.handle_iterate_opts()
+        stdout = self.get_stdout()
+        self.mock_stdout(False)
+        self.assertEqual(eb.iter_idx, 1)
+        self.assertEqual(stdout, "== starting iteration #1 ...\n")
+        self.assertEqual(eb.cfg.iterating, True)
+        self.assertEqual(eb.cfg.iterate_options, ['configopts'])
+        self.assertEqual(eb.cfg['configopts'], "--opt2")
+        self.assertEqual(eb.iter_opts, expected_iter_opts)
+
+        self.mock_stdout(True)
+        eb.handle_iterate_opts()
+        stdout = self.get_stdout()
+        self.mock_stdout(False)
+        self.assertEqual(eb.iter_idx, 2)
+        self.assertEqual(stdout, "== starting iteration #2 ...\n")
+        self.assertEqual(eb.cfg.iterating, True)
+        self.assertEqual(eb.cfg.iterate_options, ['configopts'])
+        self.assertEqual(eb.cfg['configopts'], "--opt3 --optbis")
+        self.assertEqual(eb.iter_opts, expected_iter_opts)
+
+        eb.post_iter_step()
+        self.assertEqual(eb.cfg.iterating, False)
+        self.assertEqual(eb.cfg['configopts'], ["--opt1 --anotheropt", "--opt2", "--opt3 --optbis"])
+
+    def test_extensions_step(self):
+        """Test the extensions_step"""
+        init_config(build_options={'silent': True})
+
+        self.contents = '\n'.join([
+            'easyblock = "ConfigureMake"',
+            'name = "pi"',
+            'version = "3.14"',
+            'homepage = "http://example.com"',
+            'description = "test easyconfig"',
+            'toolchain = SYSTEM',
             'exts_list = ["ext1"]',
         ])
         self.writeEC()
@@ -660,17 +786,27 @@ class EasyBlockTest(EnhancedTestCase):
 
     def test_skip_extensions_step(self):
         """Test the skip_extensions_step"""
-        self.contents = '\n'.join([
-            'easyblock = "ConfigureMake"',
-            'name = "pi"',
-            'version = "3.14"',
-            'homepage = "http://example.com"',
-            'description = "test easyconfig"',
-            'toolchain = {"name": "dummy", "version": "dummy"}',
-            'exts_list = ["ext1", "ext2"]',
-            'exts_filter = ("if [ %(ext_name)s == \'ext2\' ]; then exit 0; else exit 1; fi", "")',
-            'exts_defaultclass = "DummyExtension"',
-        ])
+        init_config(build_options={'silent': True})
+
+        self.contents = cleandoc("""
+            easyblock = "ConfigureMake"
+            name = "pi"
+            version = "3.14"
+            homepage = "http://example.com"
+            description = "test easyconfig"
+            toolchain = SYSTEM
+            exts_list = [
+                "ext1",
+                ("ext2", "42", {"source_tmpl": "dummy.tgz"}),
+                ("ext3", "1.1", {"source_tmpl": "dummy.tgz", "modulename": "real_ext"}),
+            ]
+            exts_filter = ("\
+                if [ %(ext_name)s == 'ext2' ] && [ %(ext_version)s == '42' ] && [[ %(src)s == *dummy.tgz ]];\
+                    then exit 0;\
+                elif [ %(ext_name)s == 'real_ext' ]; then exit 0;\
+                else exit 1; fi", "")
+            exts_defaultclass = "DummyExtension"
+        """)
         # check if skip skips correct extensions
         self.writeEC()
         eb = EasyBlock(EasyConfig(self.eb_file))
@@ -679,9 +815,12 @@ class EasyBlockTest(EnhancedTestCase):
         eb.skip = True
         eb.extensions_step(fetch=True)
         # 'ext1' should be in eb.exts
-        self.assertTrue('ext1' in [y for x in eb.exts for y in x.values()])
+        eb_exts = [y for x in eb.exts for y in x.values()]
+        self.assertTrue('ext1' in eb_exts)
         # 'ext2' should not
-        self.assertFalse('ext2' in [y for x in eb.exts for y in x.values()])
+        self.assertFalse('ext2' in eb_exts)
+        # 'ext3' should not
+        self.assertFalse('ext3' in eb_exts)
 
         # cleanup
         eb.close_log()
@@ -689,6 +828,12 @@ class EasyBlockTest(EnhancedTestCase):
 
     def test_make_module_step(self):
         """Test the make_module_step"""
+
+        # put dummy hidden modules in place for test123 dependency
+        test_mods = os.path.join(self.test_prefix, 'modules')
+        write_file(os.path.join(test_mods, 'test', '.1.2.3'), '#%Module')
+        self.modtool.use(test_mods)
+
         name = "pi"
         version = "3.14"
         # purposely use a 'nasty' description, that includes (unbalanced) special chars: [, ], {, }
@@ -701,18 +846,15 @@ class EasyBlockTest(EnhancedTestCase):
             'version = "%s"' % version,
             'homepage = "http://example.com"',
             'description = "%s"' % descr,
-            "toolchain = {'name': 'dummy', 'version': 'dummy'}",
-            "dependencies = [('GCC', '6.4.0-2.28'), ('toy', '0.0-deps')]",
+            "toolchain = SYSTEM",
+            "dependencies = [('GCC', '6.4.0-2.28'), ('test', '1.2.3')]",
             "builddependencies = [('OpenMPI', '2.1.2-GCC-6.4.0-2.28')]",
             # hidden deps must be included in list of (build)deps
-            "hiddendependencies = [('toy', '0.0-deps'), ('OpenMPI', '2.1.2-GCC-6.4.0-2.28')]",
+            "hiddendependencies = [('test', '1.2.3'), ('OpenMPI', '2.1.2-GCC-6.4.0-2.28')]",
             "modextravars = %s" % str(modextravars),
             "modextrapaths = %s" % str(modextrapaths),
         ])
 
-        test_dir = os.path.dirname(os.path.abspath(__file__))
-        os.environ['MODULEPATH'] = os.path.join(test_dir, 'modules')
-
         # test if module is generated correctly
         self.writeEC()
         ec = EasyConfig(self.eb_file)
@@ -773,7 +915,7 @@ class EasyBlockTest(EnhancedTestCase):
                 self.assertTrue(False, "Unknown module syntax: %s" % get_module_syntax())
             self.assertTrue(regex.search(txt), "Pattern %s found in %s" % (regex.pattern, txt))
 
-        for (name, ver) in [('toy', '0.0-deps')]:
+        for (name, ver) in [('test', '1.2.3')]:
             if get_module_syntax() == 'Tcl':
                 regex = re.compile(r'^\s*module load %s/.%s\s*$' % (name, ver), re.M)
             elif get_module_syntax() == 'Lua':
@@ -815,7 +957,7 @@ class EasyBlockTest(EnhancedTestCase):
             "version = '3.14'",
             "homepage = 'http://example.com'",
             "description = 'test easyconfig'",
-            "toolchain = {'name': 'dummy', 'version': 'dummy'}",
+            "toolchain = SYSTEM",
         ])
         self.writeEC()
         stdoutorig = sys.stdout
@@ -863,7 +1005,7 @@ class EasyBlockTest(EnhancedTestCase):
             "version = '3.14'",
             "homepage = 'http://example.com'",
             "description = 'test easyconfig'",
-            "toolchain = {'name': 'dummy', 'version': 'dummy'}",
+            "toolchain = SYSTEM",
         ])
         self.writeEC()
 
@@ -1114,7 +1256,7 @@ class EasyBlockTest(EnhancedTestCase):
             res = None
             try:
                 res = eb.obtain_file(file_url)
-            except EasyBuildError, err:
+            except EasyBuildError as err:
                 # if this fails, it should be because there's no online access
                 download_fail_regex = re.compile('socket error')
                 self.assertTrue(download_fail_regex.search(str(err)))
@@ -1128,13 +1270,31 @@ class EasyBlockTest(EnhancedTestCase):
                 eb_regex = re.compile("EasyBuild: building software with ease")
                 self.assertTrue(eb_regex.search(txt), "Pattern '%s' found in: %s" % (eb_regex.pattern, txt))
             else:
-                print "ignoring failure to download %s in test_obtain_file, testing offline?" % file_url
+                print("ignoring failure to download %s in test_obtain_file, testing offline?" % file_url)
 
         shutil.rmtree(tmpdir)
 
+    def test_obtain_file_extension(self):
+        """Test use of obtain_file method on an extension."""
+
+        testdir = os.path.abspath(os.path.dirname(__file__))
+        toy_ec_file = os.path.join(testdir, 'easyconfigs', 'test_ecs', 't', 'toy', 'toy-0.0-gompi-2018a-test.eb')
+        toy_ec = process_easyconfig(toy_ec_file)[0]
+        toy_eb = EasyBlock(toy_ec['ec'])
+
+        toy_eb.fetch_step()
+
+        test_ext = toy_eb.exts[-1]
+        test_ext_src_fn = os.path.basename(test_ext['src'])
+
+        ext = ExtensionEasyBlock(toy_eb, test_ext)
+        ext_src_path = ext.obtain_file(test_ext_src_fn)
+        self.assertEqual(os.path.basename(ext_src_path), 'toy-0.0.tar.gz')
+        self.assertTrue(os.path.exists(ext_src_path))
+
     def test_check_readiness(self):
         """Test check_readiness method."""
-        init_config(build_options={'validate': False})
+        init_config(build_options={'validate': False, 'silent': True})
 
         # check that check_readiness step works (adding dependencies, etc.)
         ec_file = 'OpenMPI-2.1.2-GCC-6.4.0-2.28.eb'
@@ -1155,7 +1315,7 @@ class EasyBlockTest(EnhancedTestCase):
         eb = EasyBlock(ec)
         try:
             eb.check_readiness_step()
-        except EasyBuildError, err:
+        except EasyBuildError as err:
             err_regex = re.compile("Missing modules dependencies .*: nosuchsoftware/1.2.3-GCC-6.4.0-2.28")
             self.assertTrue(err_regex.search(str(err)), "Pattern '%s' found in '%s'" % (err_regex.pattern, err))
 
@@ -1257,6 +1417,8 @@ class EasyBlockTest(EnhancedTestCase):
 
     def test_extensions_sanity_check(self):
         """Test sanity check aspect of extensions."""
+        init_config(build_options={'silent': True})
+
         test_ecs_dir = os.path.join(os.path.abspath(os.path.dirname(__file__)), 'easyconfigs', 'test_ecs')
         toy_ec_fn = os.path.join(test_ecs_dir, 't', 'toy', 'toy-0.0-gompi-2018a-test.eb')
 
@@ -1301,6 +1463,10 @@ class EasyBlockTest(EnhancedTestCase):
         os.close(handle)
         write_file(toy_ec2, toytxt + "\nparallel = 123\nmaxparallel = 67")
 
+        handle, toy_ec3 = tempfile.mkstemp(prefix='easyblock_test_file_', suffix='.eb')
+        os.close(handle)
+        write_file(toy_ec3, toytxt + "\nparallel = False")
+
         # default: parallellism is derived from # available cores + ulimit
         test_eb = EasyBlock(EasyConfig(toy_ec))
         test_eb.check_readiness_step()
@@ -1316,6 +1482,11 @@ class EasyBlockTest(EnhancedTestCase):
         test_eb.check_readiness_step()
         self.assertEqual(test_eb.cfg['parallel'], 67)
 
+        # make sure 'parallel = False' is not overriden (no 'parallel' build option)
+        test_eb = EasyBlock(EasyConfig(toy_ec3))
+        test_eb.check_readiness_step()
+        self.assertEqual(test_eb.cfg['parallel'], False)
+
         # only 'parallel' build option specified
         init_config(build_options={'parallel': '97', 'validate': False})
         test_eb = EasyBlock(EasyConfig(toy_ec))
@@ -1332,6 +1503,11 @@ class EasyBlockTest(EnhancedTestCase):
         test_eb.check_readiness_step()
         self.assertEqual(test_eb.cfg['parallel'], 67)
 
+        # make sure 'parallel = False' is not overriden (with 'parallel' build option)
+        test_eb = EasyBlock(EasyConfig(toy_ec3))
+        test_eb.check_readiness_step()
+        self.assertEqual(test_eb.cfg['parallel'], 0)
+
     def test_guess_start_dir(self):
         """Test guessing the start dir."""
         test_easyconfigs = os.path.join(os.path.abspath(os.path.dirname(__file__)), 'easyconfigs', 'test_ecs')
@@ -1367,7 +1543,7 @@ class EasyBlockTest(EnhancedTestCase):
         test_easyconfigs = os.path.join(os.path.abspath(os.path.dirname(__file__)), 'easyconfigs', 'test_ecs')
         ec = process_easyconfig(os.path.join(test_easyconfigs, 't', 'toy', 'toy-0.0.eb'))[0]
 
-        mkdir(os.path.join(self.test_buildpath, 'toy', '0.0', 'dummy-dummy'), parents=True)
+        mkdir(os.path.join(self.test_buildpath, 'toy', '0.0', 'system-system'), parents=True)
         eb = EasyBlock(ec['ec'])
         eb.silent = True
         eb.prepare_step()
@@ -1388,6 +1564,76 @@ class EasyBlockTest(EnhancedTestCase):
         self.assertEqual(len(self.modtool.list()), 1)
         self.assertEqual(self.modtool.list()[0]['mod_name'], 'GCC/6.4.0-2.28')
 
+    def test_prepare_step_load_tc_deps_modules(self):
+        """Test disabling loading of toolchain + dependencies in build environment."""
+
+        init_config(build_options={'robot_path': os.environ['EASYBUILD_ROBOT_PATHS']})
+
+        test_easyconfigs = os.path.join(os.path.abspath(os.path.dirname(__file__)), 'easyconfigs', 'test_ecs')
+        ompi_ec_file = os.path.join(test_easyconfigs, 'o', 'OpenMPI', 'OpenMPI-2.1.2-GCC-6.4.0-2.28.eb')
+        ec = process_easyconfig(ompi_ec_file, validate=False)[0]
+
+        mkdir(os.path.join(self.test_buildpath, 'OpenMPI', '2.1.2', 'GCC-6.4.0-2.28'), parents=True)
+        eb = EasyBlock(ec['ec'])
+        eb.silent = True
+
+        # $EBROOTGCC and $EBROOTHWLOC must be set to set up build environment
+        os.environ['EBROOTGCC'] = self.test_prefix
+        os.environ['EBROOTHWLOC'] = self.test_prefix
+
+        # loaded of modules for toolchain + dependencies can be disabled via load_tc_deps_modules=False
+        eb.prepare_step(load_tc_deps_modules=False)
+        self.assertEqual(self.modtool.list(), [])
+
+        del os.environ['EBROOTGCC']
+        del os.environ['EBROOTHWLOC']
+
+        # modules for toolchain + dependencies are still loaded by default
+        eb.prepare_step()
+        loaded_modules = self.modtool.list()
+        self.assertEqual(len(loaded_modules), 2)
+        self.assertEqual(loaded_modules[0]['mod_name'], 'GCC/6.4.0-2.28')
+        self.assertTrue(os.environ['EBROOTGCC'])
+        self.assertEqual(loaded_modules[1]['mod_name'], 'hwloc/1.11.8-GCC-6.4.0-2.28')
+        self.assertTrue(os.environ['EBROOTHWLOC'])
+
+    def test_prepare_step_hmns(self):
+        """
+        Check whether loading of already existing dependencies during prepare step works when HierarchicalMNS is used.
+        """
+        test_ecs = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'easyconfigs', 'test_ecs')
+
+        os.environ['EASYBUILD_MODULE_NAMING_SCHEME'] = 'HierarchicalMNS'
+        init_config(build_options={'robot_path': [test_ecs]})
+
+        # set up hierarchical modules, but reset $MODULEPATH to empty
+        # the expectation is that EasyBuild set's up the $MODULEPATH such that pre-installed dependencies can be loaded
+        # see also https://github.com/easybuilders/easybuild-framework/issues/2186
+        self.setup_hierarchical_modules()
+
+        self.assertTrue('GCC/6.4.0-2.28' in self.modtool.available())
+
+        self.reset_modulepath([])
+        self.assertEqual(os.environ.get('MODULEPATH'), None)
+
+        toy_ec = os.path.join(test_ecs, 't', 'toy', 'toy-0.0.eb')
+
+        test_ec = os.path.join(self.test_prefix, 'test.eb')
+        regex = re.compile('^toolchain = .*', re.M)
+        test_ectxt = regex.sub("toolchain = SYSTEM", read_file(toy_ec))
+        test_ectxt += "\ndependencies = [('GCC', '6.4.0', '-2.28')]"
+        write_file(test_ec, test_ectxt)
+
+        test_ec = process_easyconfig(test_ec)[0]
+        eb = EasyBlock(test_ec['ec'])
+
+        mkdir(os.path.join(self.test_buildpath, 'toy', '0.0', 'system-system'), parents=True)
+        eb.prepare_step()
+
+        loaded_modules = self.modtool.list()
+        self.assertEqual(len(loaded_modules), 1)
+        self.assertEqual(loaded_modules[0]['mod_name'], 'GCC/6.4.0-2.28')
+
     def test_checksum_step(self):
         """Test checksum step"""
         testdir = os.path.abspath(os.path.dirname(__file__))
@@ -1456,7 +1702,7 @@ class EasyBlockTest(EnhancedTestCase):
             expected = "Checksums missing for one or more sources/patches in toy-0.0-gompi-2018a-test.eb: "
             expected += "found 1 sources + 1 patches vs 1 checksums"
             self.assertEqual(res[0], expected)
-            self.assertTrue(res[1].startswith("Non-SHA256 checksum found for toy-0.0.tar.gz:"))
+            self.assertTrue(res[1].startswith("Non-SHA256 checksum(s) found for toy-0.0.tar.gz:"))
 
         # check for main sources/patches should reveal two issues with checksums
         res = eb.check_checksums_for(eb.cfg)
@@ -1474,6 +1720,43 @@ class EasyBlockTest(EnhancedTestCase):
             self.assertTrue(res[idx].startswith(expected))
             idx += 1
 
+        # check whether tuple of alternative SHA256 checksums is correctly recognized
+        toy_ec = os.path.join(testdir, 'easyconfigs', 'test_ecs', 't', 'toy', 'toy-0.0.eb')
+
+        ec = process_easyconfig(toy_ec)[0]
+        eb = get_easyblock_instance(ec)
+
+        # single SHA256 checksum per source/patch: OK
+        eb.cfg['checksums'] = [
+            '44332000aa33b99ad1e00cbd1a7da769220d74647060a10e807b916d73ea27bc',  # toy-0.0.tar.gz
+            '45b5e3f9f495366830e1869bb2b8f4e7c28022739ce48d9f9ebb159b439823c5',  # toy-*.patch
+            '4196b56771140d8e2468fb77f0240bc48ddbf5dabafe0713d612df7fafb1e458',  # toy-extra.txt]
+        ]
+        # no checksum issues
+        self.assertEqual(eb.check_checksums(), [])
+
+        # SHA256 checksum with type specifier: OK
+        eb.cfg['checksums'] = [
+            ('sha256', '44332000aa33b99ad1e00cbd1a7da769220d74647060a10e807b916d73ea27bc'),  # toy-0.0.tar.gz
+            '45b5e3f9f495366830e1869bb2b8f4e7c28022739ce48d9f9ebb159b439823c5',  # toy-*.patch
+            ('sha256', '4196b56771140d8e2468fb77f0240bc48ddbf5dabafe0713d612df7fafb1e458'),  # toy-extra.txt]
+        ]
+        # no checksum issues
+        self.assertEqual(eb.check_checksums(), [])
+
+        # tuple of two alternate SHA256 checksums: OK
+        eb.cfg['checksums'] = [
+            (
+                # two alternate checksums for toy-0.0.tar.gz
+                'a2848f34fcd5d6cf47def00461fcb528a0484d8edef8208d6d2e2909dc61d9cd',
+                '44332000aa33b99ad1e00cbd1a7da769220d74647060a10e807b916d73ea27bc',
+            ),
+            '45b5e3f9f495366830e1869bb2b8f4e7c28022739ce48d9f9ebb159b439823c5',  # toy-*.patch
+            '4196b56771140d8e2468fb77f0240bc48ddbf5dabafe0713d612df7fafb1e458',  # toy-extra.txt
+        ]
+        # no checksum issues
+        self.assertEqual(eb.check_checksums(), [])
+
     def test_this_is_easybuild(self):
         """Test 'this_is_easybuild' function (and get_git_revision function used by it)."""
         # make sure both return a non-Unicode string
@@ -1490,7 +1773,7 @@ class EasyBlockTest(EnhancedTestCase):
             "version = '1.0.2'",
             "homepage = 'https://example.com'",
             "description = '1st test easyconfig'",
-            "toolchain = {'name': 'dummy', 'version': ''}",
+            "toolchain = SYSTEM",
         ])
         write_file(ec1, ec1_txt)
 
@@ -1500,7 +1783,7 @@ class EasyBlockTest(EnhancedTestCase):
             "easyblock = 'Toolchain'",
             "name = 'two'",
             "version = '2.0'",
-            "toolchain = {'name': 'dummy', 'version': ''}",
+            "toolchain = SYSTEM",
             "homepage = 'https://example.com'",
             "description = '2nd test easyconfig'",
             "dependencies = [('one', '1.0')]",
@@ -1563,6 +1846,34 @@ class EasyBlockTest(EnhancedTestCase):
         self.assertEqual(hpl['class'], 'EB_HPL')
         self.assertTrue(hpl['loc'].endswith('sandbox/easybuild/easyblocks/h/hpl.py'))
 
+    def test_time2str(self):
+        """Test time2str function."""
+
+        start = datetime(2019, 7, 30, 5, 14, 23)
+
+        test_cases = [
+            (start, "0 sec"),
+            (datetime(2019, 7, 30, 5, 14, 37), "14 sec"),
+            (datetime(2019, 7, 30, 5, 15, 22), "59 sec"),
+            (datetime(2019, 7, 30, 5, 15, 23), "1 min 0 sec"),
+            (datetime(2019, 7, 30, 5, 16, 22), "1 min 59 sec"),
+            (datetime(2019, 7, 30, 5, 37, 26), "23 min 3 sec"),
+            (datetime(2019, 7, 30, 6, 14, 22), "59 min 59 sec"),
+            (datetime(2019, 7, 30, 6, 14, 23), "1 hour 0 min 0 sec"),
+            (datetime(2019, 7, 30, 6, 49, 14), "1 hour 34 min 51 sec"),
+            (datetime(2019, 7, 30, 7, 14, 23), "2 hours 0 min 0 sec"),
+            (datetime(2019, 7, 30, 8, 35, 59), "3 hours 21 min 36 sec"),
+            (datetime(2019, 7, 30, 16, 29, 24), "11 hours 15 min 1 sec"),
+            (datetime(2019, 7, 31, 5, 14, 22), "23 hours 59 min 59 sec"),
+            (datetime(2019, 7, 31, 5, 14, 23), "24 hours 0 min 0 sec"),
+            (datetime(2019, 8, 5, 20, 39, 44), "159 hours 25 min 21 sec"),
+        ]
+        for end, expected in test_cases:
+            self.assertEqual(time2str(end - start), expected)
+
+        error_pattern = "Incorrect value type provided to time2str, should be datetime.timedelta: <.* 'int'>"
+        self.assertErrorRegex(EasyBuildError, error_pattern, time2str, 123)
+
 
 def suite():
     """ return all the tests in this file """
@@ -1570,4 +1881,5 @@ def suite():
 
 
 if __name__ == '__main__':
-    TextTestRunner(verbosity=1).run(suite())
+    res = TextTestRunner(verbosity=1).run(suite())
+    sys.exit(len(res.failures))
