@@ -1,5 +1,5 @@
 # #
-# Copyright 2012-2019 Ghent University
+# Copyright 2012-2020 Ghent University
 #
 # This file is part of EasyBuild,
 # originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
@@ -38,14 +38,13 @@ import shutil
 import stat
 import sys
 import tempfile
-import urllib2
 from test.framework.utilities import EnhancedTestCase, TestLoaderFiltered, init_config
 from unittest import TextTestRunner
-from urllib2 import URLError
 
 import easybuild.tools.filetools as ft
 from easybuild.tools.build_log import EasyBuildError
 from easybuild.tools.multidiff import multidiff
+from easybuild.tools.py2vs3 import std_urllib
 
 
 class FileToolsTest(EnhancedTestCase):
@@ -63,13 +62,13 @@ class FileToolsTest(EnhancedTestCase):
         """Test setup."""
         super(FileToolsTest, self).setUp()
 
-        self.orig_filetools_urllib2_urlopen = ft.urllib2.urlopen
+        self.orig_filetools_std_urllib_urlopen = ft.std_urllib.urlopen
 
     def tearDown(self):
         """Cleanup."""
         super(FileToolsTest, self).tearDown()
 
-        ft.urllib2.urlopen = self.orig_filetools_urllib2_urlopen
+        ft.std_urllib.urlopen = self.orig_filetools_std_urllib_urlopen
 
     def test_extract_cmd(self):
         """Test various extract commands."""
@@ -195,18 +194,18 @@ class FileToolsTest(EnhancedTestCase):
         # put a directory 'foo' in place (should be ignored by 'which')
         foo = os.path.join(self.test_prefix, 'foo')
         ft.mkdir(foo)
-        ft.adjust_permissions(foo, stat.S_IRUSR|stat.S_IXUSR)
+        ft.adjust_permissions(foo, stat.S_IRUSR | stat.S_IXUSR)
         # put executable file 'bar' in place
         bar = os.path.join(self.test_prefix, 'bar')
         ft.write_file(bar, '#!/bin/bash')
-        ft.adjust_permissions(bar, stat.S_IRUSR|stat.S_IXUSR)
+        ft.adjust_permissions(bar, stat.S_IRUSR | stat.S_IXUSR)
         self.assertEqual(ft.which('foo'), None)
         self.assertTrue(os.path.samefile(ft.which('bar'), bar))
 
         # add another location to 'bar', which should only return the first location by default
         barbis = os.path.join(self.test_prefix, 'more', 'bar')
         ft.write_file(barbis, '#!/bin/bash')
-        ft.adjust_permissions(barbis, stat.S_IRUSR|stat.S_IXUSR)
+        ft.adjust_permissions(barbis, stat.S_IRUSR | stat.S_IXUSR)
         os.environ['PATH'] = '%s:%s' % (os.environ['PATH'], os.path.dirname(barbis))
         self.assertTrue(os.path.samefile(ft.which('bar'), bar))
 
@@ -216,18 +215,39 @@ class FileToolsTest(EnhancedTestCase):
         self.assertTrue(os.path.samefile(res[0], bar))
         self.assertTrue(os.path.samefile(res[1], barbis))
 
+        # both read/exec permissions must be available
+        # if read permissions are removed for first hit, second hit is found instead
+        ft.adjust_permissions(bar, stat.S_IRUSR, add=False)
+        self.assertTrue(os.path.samefile(ft.which('bar'), barbis))
+
+        # likewise for read permissions
+        ft.adjust_permissions(bar, stat.S_IRUSR, add=True)
+        self.assertTrue(os.path.samefile(ft.which('bar'), bar))
+
+        ft.adjust_permissions(bar, stat.S_IXUSR, add=False)
+        self.assertTrue(os.path.samefile(ft.which('bar'), barbis))
+
+        # if read permission on other 'bar' are also removed, nothing is found anymore
+        ft.adjust_permissions(barbis, stat.S_IRUSR, add=False)
+        self.assertEqual(ft.which('bar'), None)
+
+        # checking of read/exec permissions can be disabled via 'check_perms'
+        self.assertTrue(os.path.samefile(ft.which('bar', check_perms=False), bar))
+
     def test_checksums(self):
         """Test checksum functionality."""
-        fh, fp = tempfile.mkstemp()
-        os.close(fh)
+
+        fp = os.path.join(self.test_prefix, 'test.txt')
         ft.write_file(fp, "easybuild\n")
+
         known_checksums = {
             'adler32': '0x379257805',
             'crc32': '0x1457143216',
             'md5': '7167b64b1ca062b9674ffef46f9325db',
             'sha1': 'db05b79e09a4cc67e9dd30b313b5488813db3190',
             'sha256': '1c49562c4b404f3120a3fa0926c8d09c99ef80e470f7de03ffdfa14047960ea5',
-            'sha512': '7610f6ce5e91e56e350d25c917490e4815f7986469fafa41056698aec256733eb7297da8b547d5e74b851d7c4e475900cec4744df0f887ae5c05bf1757c224b4',
+            'sha512': '7610f6ce5e91e56e350d25c917490e4815f7986469fafa41056698aec256733e'
+                      'b7297da8b547d5e74b851d7c4e475900cec4744df0f887ae5c05bf1757c224b4',
         }
 
         # make sure checksums computation/verification is correct
@@ -242,10 +262,11 @@ class FileToolsTest(EnhancedTestCase):
         self.assertTrue(ft.verify_checksum(fp, known_checksums['md5']))
         self.assertTrue(ft.verify_checksum(fp, known_checksums['sha256']))
 
-        # checksum of length 32 is assumed to be MD5, length 64 to be SHA256, other lengths not allowed
         # providing non-matching MD5 and SHA256 checksums results in failed verification
         self.assertFalse(ft.verify_checksum(fp, '1c49562c4b404f3120a3fa0926c8d09c'))
         self.assertFalse(ft.verify_checksum(fp, '7167b64b1ca062b9674ffef46f9325db7167b64b1ca062b9674ffef46f9325db'))
+
+        # checksum of length 32 is assumed to be MD5, length 64 to be SHA256, other lengths not allowed
         # checksum of length other than 32/64 yields an error
         error_pattern = "Length of checksum '.*' \(\d+\) does not match with either MD5 \(32\) or SHA256 \(64\)"
         for checksum in ['tooshort', 'inbetween32and64charactersisnotgoodeither', known_checksums['sha256'] + 'foo']:
@@ -261,6 +282,23 @@ class FileToolsTest(EnhancedTestCase):
         self.assertFalse(ft.verify_checksum(fp, broken_checksums['md5']))
         self.assertFalse(ft.verify_checksum(fp, broken_checksums['sha256']))
 
+        # test specify alternative checksums
+        alt_checksums = ('7167b64b1ca062b9674ffef46f9325db7167b64b1ca062b9674ffef46f9325db', known_checksums['sha256'])
+        self.assertTrue(ft.verify_checksum(fp, alt_checksums))
+
+        alt_checksums = ('fecf50db81148786647312bbd3b5c740', '2c829facaba19c0fcd81f9ce96bef712',
+                         '840078aeb4b5d69506e7c8edae1e1b89', known_checksums['md5'])
+        self.assertTrue(ft.verify_checksum(fp, alt_checksums))
+
+        alt_checksums = ('840078aeb4b5d69506e7c8edae1e1b89', known_checksums['md5'], '2c829facaba19c0fcd81f9ce96bef712')
+        self.assertTrue(ft.verify_checksum(fp, alt_checksums))
+
+        alt_checksums = (known_checksums['md5'], '840078aeb4b5d69506e7c8edae1e1b89', '2c829facaba19c0fcd81f9ce96bef712')
+        self.assertTrue(ft.verify_checksum(fp, alt_checksums))
+
+        alt_checksums = (known_checksums['sha256'],)
+        self.assertTrue(ft.verify_checksum(fp, alt_checksums))
+
         # check whether missing checksums are enforced
         build_options = {
             'enforce_checksums': True,
@@ -271,8 +309,10 @@ class FileToolsTest(EnhancedTestCase):
         self.assertTrue(ft.verify_checksum(fp, known_checksums['md5']))
         self.assertTrue(ft.verify_checksum(fp, known_checksums['sha256']))
 
-        # cleanup
-        os.remove(fp)
+        # Test dictionary-type checksums
+        for checksum in [known_checksums[x] for x in ('md5', 'sha256')]:
+            dict_checksum = {os.path.basename(fp): checksum, 'foo': 'baa'}
+            self.assertTrue(ft.verify_checksum(fp, dict_checksum))
 
     def test_common_path_prefix(self):
         """Test get common path prefix for a list of paths."""
@@ -302,9 +342,9 @@ class FileToolsTest(EnhancedTestCase):
         self.assertEqual(ft.download_file(fn, 'file://%s/nosuchfile' % test_dir, target_location), None)
 
         # install broken proxy handler for opening local files
-        # this should make urllib2.urlopen use this broken proxy for downloading from a file:// URL
-        proxy_handler = urllib2.ProxyHandler({'file': 'file://%s/nosuchfile' % test_dir})
-        urllib2.install_opener(urllib2.build_opener(proxy_handler))
+        # this should make urlopen use this broken proxy for downloading from a file:// URL
+        proxy_handler = std_urllib.ProxyHandler({'file': 'file://%s/nosuchfile' % test_dir})
+        std_urllib.install_opener(std_urllib.build_opener(proxy_handler))
 
         # downloading over a broken proxy results in None return value (failed download)
         # this tests whether proxies are taken into account by download_file
@@ -314,7 +354,7 @@ class FileToolsTest(EnhancedTestCase):
         ft.write_file(target_location, '')
 
         # restore a working file handler, and retest download of local file
-        urllib2.install_opener(urllib2.build_opener(urllib2.FileHandler()))
+        std_urllib.install_opener(std_urllib.build_opener(std_urllib.FileHandler()))
         res = ft.download_file(fn, source_url, target_location)
         self.assertEqual(res, target_location, "'download' of local file works after removing broken proxy")
 
@@ -331,11 +371,11 @@ class FileToolsTest(EnhancedTestCase):
         target_location = os.path.join(self.test_prefix, 'jenkins_robots.txt')
         url = 'https://raw.githubusercontent.com/easybuilders/easybuild-framework/master/README.rst'
         try:
-            urllib2.urlopen(url)
+            std_urllib.urlopen(url)
             res = ft.download_file(fn, url, target_location)
             self.assertEqual(res, target_location, "download with specified timeout works")
-        except urllib2.URLError:
-            print "Skipping timeout test in test_download_file (working offline)"
+        except std_urllib.URLError:
+            print("Skipping timeout test in test_download_file (working offline)")
 
         # also test behaviour of download_file under --dry-run
         build_options = {
@@ -367,13 +407,30 @@ class FileToolsTest(EnhancedTestCase):
         fn = 'README.rst'
         target = os.path.join(self.test_prefix, fn)
 
-        # replace urllib2.urlopen with function that raises SSL error
-        def fake_urllib2_open(*args, **kwargs):
+        # replaceurlopen with function that raises SSL error
+        def fake_urllib_open(*args, **kwargs):
             error_msg = "<urlopen error [Errno 1] _ssl.c:510: error:12345:"
             error_msg += "SSL routines:SSL23_GET_SERVER_HELLO:sslv3 alert handshake failure>"
             raise IOError(error_msg)
 
-        ft.urllib2.urlopen = fake_urllib2_open
+        ft.std_urllib.urlopen = fake_urllib_open
+
+        # if requests is available, file is downloaded
+        if ft.HAVE_REQUESTS:
+            res = ft.download_file(fn, url, target)
+            self.assertTrue(res and os.path.exists(res))
+            self.assertTrue("https://easybuilders.github.io/easybuild" in ft.read_file(res))
+
+        # without requests being available, error is raised
+        ft.HAVE_REQUESTS = False
+        self.assertErrorRegex(EasyBuildError, "SSL issues with urllib2", ft.download_file, fn, url, target)
+
+        # replaceurlopen with function that raises HTTP error 403
+        def fake_urllib_open(*args, **kwargs):
+            from easybuild.tools.py2vs3 import StringIO
+            raise ft.std_urllib.HTTPError(url, 403, "Forbidden", "", StringIO())
+
+        ft.std_urllib.urlopen = fake_urllib_open
 
         # if requests is available, file is downloaded
         if ft.HAVE_REQUESTS:
@@ -501,7 +558,7 @@ class FileToolsTest(EnhancedTestCase):
 
         # creating the symlink
         link = os.path.join(self.test_prefix, 'test.link')
-        ft.symlink(fp, link) # test if is symlink is valid is done elsewhere
+        ft.symlink(fp, link)  # test if is symlink is valid is done elsewhere
 
         # Attempting to remove a valid symlink
         ft.remove_file(link)
@@ -596,6 +653,9 @@ class FileToolsTest(EnhancedTestCase):
         self.assertTrue(os.path.exists(foo))
         self.assertEqual(ft.read_file(foo), 'bar')
 
+        # test use of 'mode' in read_file
+        self.assertEqual(ft.read_file(foo, mode='rb'), b'bar')
+
     def test_det_patched_files(self):
         """Test det_patched_files function."""
         toy_patch_fn = 'toy-0.0_fix-silly-typo-in-printf-statement.patch'
@@ -773,8 +833,8 @@ class FileToolsTest(EnhancedTestCase):
         ft.write_file(fp + '.1', 'evenmoarbar')
         ft.move_logs(fp, os.path.join(self.test_prefix, 'bar.log'))
 
-        logs = sorted([f for f in os.listdir(self.test_prefix) if 'log' in f])
-        self.assertEqual(len(logs), 7)
+        logs = sorted([f for f in os.listdir(self.test_prefix) if '.log' in f])
+        self.assertEqual(len(logs), 7, "Found exactly 7 log files: %d (%s)" % (len(logs), logs))
         self.assertEqual(len([x for x in logs if x.startswith('eb-test-')]), 1)
         self.assertEqual(len([x for x in logs if x.startswith('foo')]), 2)
         self.assertEqual(len([x for x in logs if x.startswith('bar')]), 4)
@@ -806,17 +866,15 @@ class FileToolsTest(EnhancedTestCase):
         self.assertEqual(lines[1], "=====")
 
         # different versionsuffix
-        self.assertTrue(lines[2].startswith("3 %s- versionsuffix = '-test'%s (1/2) toy-0.0-" % (red, endcol)))
-        self.assertTrue(lines[3].startswith("3 %s- versionsuffix = '-deps'%s (1/2) toy-0.0-" % (red, endcol)))
+        self.assertTrue(lines[2].startswith("3 %s- versionsuffix = '-deps'%s (1/2) toy-0.0-" % (red, endcol)))
+        self.assertTrue(lines[3].startswith("3 %s- versionsuffix = '-test'%s (1/2) toy-0.0-" % (red, endcol)))
 
-        # different toolchain in toy-0.0-gompi-1.3.12-test: '+' line (removed chars in toolchain name/version, in red)
-        expected = "7 %(endcol)s-%(endcol)s toolchain = {"
-        expected += "'name': '%(endcol)s%(red)sgo%(endcol)sm\x1b[0m%(red)spi%(endcol)s', "
+        # different toolchain in toy-0.0-gompi-1.3.12-test: '+' line (added line in green)
+        expected = "7 %(green)s+ toolchain = SYSTEM%(endcol)s"
         expected = expected % {'endcol': endcol, 'green': green, 'red': red}
         self.assertTrue(lines[7].startswith(expected))
-        # different toolchain in toy-0.0-gompi-1.3.12-test: '+' line (added chars in toolchain name/version, in green)
-        expected = "7 %(endcol)s+%(endcol)s toolchain = {"
-        expected += "'name': '%(endcol)s%(green)sdu%(endcol)sm\x1b[0m%(green)smy%(endcol)s', "
+        # different toolchain in toy-0.0-gompi-1.3.12-test: '-' line (removed line in red)
+        expected = "8 %(red)s- toolchain = {'name': 'gompi', 'version': '2018a'}%(endcol)s"
         expected = expected % {'endcol': endcol, 'green': green, 'red': red}
         self.assertTrue(lines[8].startswith(expected))
 
@@ -832,19 +890,14 @@ class FileToolsTest(EnhancedTestCase):
         self.assertEqual(lines[1], "=====")
 
         # different versionsuffix
-        self.assertTrue(lines[2].startswith("3 - versionsuffix = '-test' (1/2) toy-0.0-"))
-        self.assertTrue(lines[3].startswith("3 - versionsuffix = '-deps' (1/2) toy-0.0-"))
+        self.assertTrue(lines[2].startswith("3 - versionsuffix = '-deps' (1/2) toy-0.0-"))
+        self.assertTrue(lines[3].startswith("3 - versionsuffix = '-test' (1/2) toy-0.0-"))
 
-        # different toolchain in toy-0.0-gompi-2018a-test: '+' line with squigly line underneath to mark removed chars
-        expected = "7 - toolchain = {'name': 'gompi', 'version': '2018a'} (1/2) toy"
+        # different toolchain in toy-0.0-gompi-2018a-test: '+' added line, '-' removed line
+        expected = "7 + toolchain = SYSTEM (1/2) toy"
         self.assertTrue(lines[7].startswith(expected))
-        expected = "  ?                       ^^ ^^ "
+        expected = "8 - toolchain = {'name': 'gompi', 'version': '2018a'} (1/2) toy"
         self.assertTrue(lines[8].startswith(expected))
-        # different toolchain in toy-0.0-gompi-2018a-test: '-' line with squigly line underneath to mark added chars
-        expected = "7 + toolchain = {'name': 'dummy', 'version': 'dummy'} (1/2) toy"
-        self.assertTrue(lines[9].startswith(expected))
-        expected = "  ?                       ^^ ^^ "
-        self.assertTrue(lines[10].startswith(expected))
 
         # no postinstallcmds in toy-0.0-deps.eb
         expected = "29 + postinstallcmds = "
@@ -906,7 +959,7 @@ class FileToolsTest(EnhancedTestCase):
     def test_adjust_permissions(self):
         """Test adjust_permissions"""
         # set umask hard to run test reliably
-        orig_umask = os.umask(0022)
+        orig_umask = os.umask(0o022)
 
         # prep files/dirs/(broken) symlinks is test dir
 
@@ -947,7 +1000,7 @@ class FileToolsTest(EnhancedTestCase):
         # test default behaviour:
         # recursive, add permissions, relative to existing permissions, both files and dirs, skip symlinks
         # add user execution, group write permissions
-        ft.adjust_permissions(self.test_prefix, stat.S_IXUSR|stat.S_IWGRP)
+        ft.adjust_permissions(self.test_prefix, stat.S_IXUSR | stat.S_IWGRP)
 
         # foo file: rwxrw-r--
         foo_perms = os.stat(os.path.join(self.test_prefix, 'foo'))[stat.ST_MODE]
@@ -971,6 +1024,30 @@ class FileToolsTest(EnhancedTestCase):
             for bit in [stat.S_IXGRP, stat.S_IWOTH, stat.S_IXOTH]:
                 self.assertFalse(perms & bit)
 
+        # check error reporting when changing permissions fails
+        nosuchdir = os.path.join(self.test_prefix, 'nosuchdir')
+        err_msg = "Failed to chmod/chown several paths.*No such file or directory"
+        self.assertErrorRegex(EasyBuildError, err_msg, ft.adjust_permissions, nosuchdir, stat.S_IWOTH)
+        nosuchfile = os.path.join(self.test_prefix, 'nosuchfile')
+        self.assertErrorRegex(EasyBuildError, err_msg, ft.adjust_permissions, nosuchfile, stat.S_IWUSR, recursive=False)
+
+        # try using adjust_permissions on a file not owned by current user,
+        # using permissions that are actually already correct;
+        # actual chmod should be skipped, otherwise it fails (you need to own a file to change permissions on it)
+
+        # use /bin/ls, which should always be there, has read/exec permissions for anyone (755), and is owned by root
+        ls_path = '/bin/ls'
+
+        # try adding read/exec permissions for current user (which is already there)
+        ft.adjust_permissions(ls_path, stat.S_IRUSR | stat.S_IXUSR, add=True)
+
+        # try removing write permissions for others (which are not set already)
+        ft.adjust_permissions(ls_path, stat.S_IWOTH, add=False)
+
+        # try hard setting permissions using current permissions
+        current_ls_perms = os.stat(ls_path)[stat.ST_MODE]
+        ft.adjust_permissions(ls_path, current_ls_perms, relative=False)
+
         # restore original umask
         os.umask(orig_umask)
 
@@ -985,7 +1062,7 @@ class FileToolsTest(EnhancedTestCase):
             ft.symlink(test_files[-1], os.path.join(testdir, 'symlink%s' % idx))
 
         # by default, 50% of failures are allowed (to be robust against failures to change permissions)
-        perms = stat.S_IRUSR|stat.S_IWUSR|stat.S_IXUSR
+        perms = stat.S_IRUSR | stat.S_IWUSR | stat.S_IXUSR
 
         ft.adjust_permissions(testdir, perms, recursive=True, ignore_errors=True)
 
@@ -1047,6 +1124,35 @@ class FileToolsTest(EnhancedTestCase):
         new_testtxt = ft.read_file(testfile)
         self.assertEqual(new_testtxt, expected_testtxt)
 
+        # backup file is created by default
+        backup = testfile + '.orig.eb'
+        self.assertTrue(os.path.exists(backup))
+        self.assertEqual(ft.read_file(backup), testtxt)
+
+        # cleanup
+        ft.remove_file(backup)
+        ft.write_file(testfile, testtxt)
+
+        # extension of backed up file can be controlled
+        ft.apply_regex_substitutions(testfile, regex_subs, backup='.backup')
+
+        new_testtxt = ft.read_file(testfile)
+        self.assertEqual(new_testtxt, expected_testtxt)
+
+        backup = testfile + '.backup'
+        self.assertTrue(os.path.exists(backup))
+        self.assertEqual(ft.read_file(backup), testtxt)
+
+        # cleanup
+        ft.remove_file(backup)
+        ft.write_file(testfile, testtxt)
+
+        # creation of backup can be avoided
+        ft.apply_regex_substitutions(testfile, regex_subs, backup=False)
+        new_testtxt = ft.read_file(testfile)
+        self.assertEqual(new_testtxt, expected_testtxt)
+        self.assertFalse(os.path.exists(backup))
+
         # passing empty list of substitions is a no-op
         ft.write_file(testfile, testtxt)
         ft.apply_regex_substitutions(testfile, [])
@@ -1218,7 +1324,7 @@ class FileToolsTest(EnhancedTestCase):
         self.assertErrorRegex(EasyBuildError, "Couldn't apply patch file", ft.apply_patch, toy_patch, path)
 
     def test_copy_file(self):
-        """ Test copy_file """
+        """Test copy_file function."""
         testdir = os.path.dirname(os.path.abspath(__file__))
         to_copy = os.path.join(testdir, 'easyconfigs', 'test_ecs', 't', 'toy', 'toy-0.0.eb')
         target_path = os.path.join(self.test_prefix, 'toy.eb')
@@ -1230,6 +1336,27 @@ class FileToolsTest(EnhancedTestCase):
         src, target = os.path.dirname(to_copy), os.path.join(self.test_prefix, 'toy')
         self.assertErrorRegex(EasyBuildError, "Failed to copy file.*Is a directory", ft.copy_file, src, target)
 
+        # test overwriting of existing file owned by someone else,
+        # which should make copy_file use shutil.copyfile rather than shutil.copy2
+        test_file_contents = "This is just a test, 1, 2, 3, check"
+        test_file_to_copy = os.path.join(self.test_prefix, 'test123.txt')
+        ft.write_file(test_file_to_copy, test_file_contents)
+
+        # this test file must be created before, we can't create a file owned by another account
+        test_file_to_overwrite = os.path.join('/tmp', 'file_to_overwrite_for_easybuild_test_copy_file.txt')
+        if os.path.exists(test_file_to_overwrite):
+            # make sure target file is owned by another user (we don't really care who)
+            self.assertTrue(os.stat(test_file_to_overwrite).st_uid != os.getuid())
+            # make sure the target file is writeable by current user (otherwise the copy will definitely fail)
+            self.assertTrue(os.access(test_file_to_overwrite, os.W_OK))
+
+            ft.copy_file(test_file_to_copy, test_file_to_overwrite)
+            self.assertEqual(ft.read_file(test_file_to_overwrite), test_file_contents)
+        else:
+            # printing this message will make test suite fail in Travis/GitHub CI,
+            # since we check for unexpected output produced by the tests
+            print("Skipping overwrite-file-owned-by-other-user copy_file test (%s is missing)", test_file_to_overwrite)
+
         # also test behaviour of copy_file under --dry-run
         build_options = {
             'extended_dry_run': True,
@@ -1258,8 +1385,44 @@ class FileToolsTest(EnhancedTestCase):
         self.assertTrue(ft.read_file(to_copy) == ft.read_file(target_path))
         self.assertEqual(txt, '')
 
+    def test_copy_files(self):
+        """Test copy_files function."""
+        test_ecs = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'easyconfigs', 'test_ecs')
+        toy_ec = os.path.join(test_ecs, 't', 'toy', 'toy-0.0.eb')
+        toy_ec_txt = ft.read_file(toy_ec)
+        bzip2_ec = os.path.join(test_ecs, 'b', 'bzip2', 'bzip2-1.0.6-GCC-4.9.2.eb')
+        bzip2_ec_txt = ft.read_file(bzip2_ec)
+
+        # copying a single file to a non-existing directory
+        target_dir = os.path.join(self.test_prefix, 'target_dir1')
+        ft.copy_files([toy_ec], target_dir)
+        copied_toy_ec = os.path.join(target_dir, 'toy-0.0.eb')
+        self.assertTrue(os.path.exists(copied_toy_ec))
+        self.assertEqual(ft.read_file(copied_toy_ec), toy_ec_txt)
+
+        # copying a single file to an existing directory
+        ft.copy_files([bzip2_ec], target_dir)
+        copied_bzip2_ec = os.path.join(target_dir, 'bzip2-1.0.6-GCC-4.9.2.eb')
+        self.assertTrue(os.path.exists(copied_bzip2_ec))
+        self.assertEqual(ft.read_file(copied_bzip2_ec), bzip2_ec_txt)
+
+        # copying multiple files to a non-existing directory
+        target_dir = os.path.join(self.test_prefix, 'target_dir_multiple')
+        ft.copy_files([toy_ec, bzip2_ec], target_dir)
+        copied_toy_ec = os.path.join(target_dir, 'toy-0.0.eb')
+        self.assertTrue(os.path.exists(copied_toy_ec))
+        self.assertEqual(ft.read_file(copied_toy_ec), toy_ec_txt)
+        copied_bzip2_ec = os.path.join(target_dir, 'bzip2-1.0.6-GCC-4.9.2.eb')
+        self.assertTrue(os.path.exists(copied_bzip2_ec))
+        self.assertEqual(ft.read_file(copied_bzip2_ec), bzip2_ec_txt)
+
+        # copying files to an existing target that is not a directory results in an error
+        self.assertTrue(os.path.isfile(copied_toy_ec))
+        error_pattern = "/toy-0.0.eb exists but is not a directory"
+        self.assertErrorRegex(EasyBuildError, error_pattern, ft.copy_files, [bzip2_ec], copied_toy_ec)
+
     def test_copy_dir(self):
-        """Test copy_file"""
+        """Test copy_dir function."""
         testdir = os.path.dirname(os.path.abspath(__file__))
         to_copy = os.path.join(testdir, 'easyconfigs', 'test_ecs', 'g', 'GCC')
 
@@ -1456,7 +1619,7 @@ class FileToolsTest(EnhancedTestCase):
         # check error handling (after creating a permission problem with removing files/dirs)
         ft.write_file(testfile, 'bar')
         ft.mkdir(test_dir)
-        ft.adjust_permissions(self.test_prefix, stat.S_IWUSR|stat.S_IWGRP|stat.S_IWOTH, add=False)
+        ft.adjust_permissions(self.test_prefix, stat.S_IWUSR | stat.S_IWGRP | stat.S_IWOTH, add=False)
         self.assertErrorRegex(EasyBuildError, "Failed to remove", ft.remove_file, testfile)
         self.assertErrorRegex(EasyBuildError, "Failed to remove", ft.remove, testfile)
         self.assertErrorRegex(EasyBuildError, "Failed to remove", ft.remove_dir, test_dir)
@@ -1541,6 +1704,50 @@ class FileToolsTest(EnhancedTestCase):
                                 'hwloc-1.11.8-GCC-7.3.0-2.30.eb', 'hwloc-1.6.2-GCC-4.9.3-2.26.eb',
                                 'hwloc-1.8-gcccuda-2018a.eb'])
 
+        # patterns that include special characters + (or ++) shouldn't cause trouble
+        # cfr. https://github.com/easybuilders/easybuild-framework/issues/2966
+        for pattern in ['netCDF-C++', 'foo.*bar', 'foo|bar']:
+            var_defs, hits = ft.search_file([test_ecs], pattern, terse=True, filename_only=True)
+            self.assertEqual(var_defs, [])
+            # no hits for any of these in test easyconfigs
+            self.assertEqual(hits, [])
+
+        # create hit for netCDF-C++ search
+        test_ec = os.path.join(self.test_prefix, 'netCDF-C++-4.2-foss-2019a.eb')
+        ft.write_file(test_ec, '')
+        for pattern in ['netCDF-C++', 'CDF', 'C++', '^netCDF']:
+            var_defs, hits = ft.search_file([self.test_prefix], pattern, terse=True, filename_only=True)
+            self.assertEqual(var_defs, [])
+            self.assertEqual(hits, ['netCDF-C++-4.2-foss-2019a.eb'])
+
+        # check how simply invalid queries are handled
+        for pattern in ['*foo', '(foo', ')foo', 'foo)', 'foo(']:
+            self.assertErrorRegex(EasyBuildError, "Invalid search query", ft.search_file, [test_ecs], pattern)
+
+    def test_dir_contains_files(self):
+        def makedirs_in_test(*paths):
+            """Make dir specified by paths and return top-level folder"""
+            os.makedirs(os.path.join(self.test_prefix, *paths))
+            return os.path.join(self.test_prefix, paths[0])
+
+        empty_dir = makedirs_in_test('empty_dir')
+        self.assertFalse(ft.dir_contains_files(empty_dir))
+
+        dir_w_subdir = makedirs_in_test('dir_w_subdir', 'sub_dir')
+        self.assertFalse(ft.dir_contains_files(dir_w_subdir))
+
+        dir_subdir_file = makedirs_in_test('dir_subdir_file', 'sub_dir_w_file')
+        ft.write_file(os.path.join(dir_subdir_file, 'sub_dir_w_file', 'file.h'), '')
+        self.assertTrue(ft.dir_contains_files(dir_subdir_file))
+
+        dir_w_file = makedirs_in_test('dir_w_file')
+        ft.write_file(os.path.join(dir_w_file, 'file.h'), '')
+        self.assertTrue(ft.dir_contains_files(dir_w_file))
+
+        dir_w_dir_and_file = makedirs_in_test('dir_w_dir_and_file', 'sub_dir')
+        ft.write_file(os.path.join(dir_w_dir_and_file, 'file.h'), '')
+        self.assertTrue(ft.dir_contains_files(dir_w_dir_and_file))
+
     def test_find_eb_script(self):
         """Test find_eb_script function."""
         self.assertTrue(os.path.exists(ft.find_eb_script('rpath_args.py')))
@@ -1607,7 +1814,7 @@ class FileToolsTest(EnhancedTestCase):
         ft.write_file(test_file, 'foo')
 
         # timestamp should be exactly 14 digits (year, month, day, hours, minutes, seconds)
-        regex = re.compile('^test\.txt_[0-9]{14}$')
+        regex = re.compile(r'^test\.txt_[0-9]{14}_[0-9]+$')
 
         res = ft.find_backup_name_candidate(test_file)
         self.assertTrue(os.path.samefile(os.path.dirname(res), self.test_prefix))
@@ -1670,20 +1877,20 @@ class FileToolsTest(EnhancedTestCase):
         target_dir = os.path.join(self.test_prefix, 'target')
 
         try:
-            res = ft.get_source_tarball_from_git('test.tar.gz', target_dir, git_config)
+            ft.get_source_tarball_from_git('test.tar.gz', target_dir, git_config)
             # (only) tarball is created in specified target dir
             self.assertTrue(os.path.isfile(os.path.join(target_dir, 'test.tar.gz')))
             self.assertEqual(os.listdir(target_dir), ['test.tar.gz'])
 
             del git_config['tag']
             git_config['commit'] = '8456f86'
-            res = ft.get_source_tarball_from_git('test2.tar.gz', target_dir, git_config)
+            ft.get_source_tarball_from_git('test2.tar.gz', target_dir, git_config)
             self.assertTrue(os.path.isfile(os.path.join(target_dir, 'test2.tar.gz')))
             self.assertEqual(sorted(os.listdir(target_dir)), ['test.tar.gz', 'test2.tar.gz'])
 
         except EasyBuildError as err:
             if "Network is down" in str(err):
-                print "Ignoring download error in test_get_source_tarball_from_git, working offline?"
+                print("Ignoring download error in test_get_source_tarball_from_git, working offline?")
             else:
                 raise err
 
@@ -1802,10 +2009,69 @@ class FileToolsTest(EnhancedTestCase):
         ]:
             self.assertFalse(ft.is_sha256_checksum(not_a_sha256_checksum))
 
+    def test_fake_vsc(self):
+        """Test whether importing from 'vsc.*' namespace results in an error after calling install_fake_vsc."""
+
+        ft.install_fake_vsc()
+
+        self.mock_stderr(True)
+        self.mock_stdout(True)
+        try:
+            import vsc  # noqa
+            self.assertTrue(False, "'import vsc' results in an error")
+        except SystemExit:
+            pass
+
+        stderr = self.get_stderr()
+        stdout = self.get_stdout()
+        self.mock_stderr(False)
+        self.mock_stdout(False)
+
+        self.assertEqual(stdout, '')
+
+        error_pattern = r"Detected import from 'vsc' namespace in .*test/framework/filetools.py \(line [0-9]+\)"
+        regex = re.compile(r"^\nERROR: %s" % error_pattern)
+        self.assertTrue(regex.search(stderr), "Pattern '%s' found in: %s" % (regex.pattern, stderr))
+
+        # also test with import from another module
+        test_python_mod = os.path.join(self.test_prefix, 'test_fake_vsc', 'import_vsc.py')
+        ft.write_file(os.path.join(os.path.dirname(test_python_mod), '__init__.py'), '')
+        ft.write_file(test_python_mod, 'import vsc')
+
+        sys.path.insert(0, self.test_prefix)
+
+        self.mock_stderr(True)
+        self.mock_stdout(True)
+        try:
+            from test_fake_vsc import import_vsc  # noqa
+            self.assertTrue(False, "'import vsc' results in an error")
+        except SystemExit:
+            pass
+        stderr = self.get_stderr()
+        stdout = self.get_stdout()
+        self.mock_stderr(False)
+        self.mock_stdout(False)
+
+        self.assertEqual(stdout, '')
+        error_pattern = r"Detected import from 'vsc' namespace in .*/test_fake_vsc/import_vsc.py \(line 1\)"
+        regex = re.compile(r"^\nERROR: %s" % error_pattern)
+        self.assertTrue(regex.search(stderr), "Pattern '%s' found in: %s" % (regex.pattern, stderr))
+
+        # no error if import was detected from pkgutil.py,
+        # since that may be triggered by a system-wide vsc-base installation
+        # (even though no code is doing 'import vsc'...)
+        ft.move_file(test_python_mod, os.path.join(os.path.dirname(test_python_mod), 'pkgutil.py'))
+
+        from test_fake_vsc import pkgutil
+        self.assertTrue(pkgutil.__file__.endswith('/test_fake_vsc/pkgutil.py'))
+
+
 
 def suite():
     """ returns all the testcases in this module """
     return TestLoaderFiltered().loadTestsFromTestCase(FileToolsTest, sys.argv[1:])
 
+
 if __name__ == '__main__':
-    TextTestRunner(verbosity=1).run(suite())
+    res = TextTestRunner(verbosity=1).run(suite())
+    sys.exit(len(res.failures))
