@@ -1,5 +1,5 @@
 ##
-# Copyright 2012-2019 Ghent University
+# Copyright 2012-2020 Ghent University
 #
 # This file is part of EasyBuild,
 # originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
@@ -30,17 +30,15 @@ Unit tests for module_generator.py.
 """
 import glob
 import os
+import re
 import sys
 import tempfile
 from distutils.version import LooseVersion
 from unittest import TextTestRunner, TestSuite
-from vsc.utils.fancylogger import setLogLevelDebug, logToScreen
-from vsc.utils.missing import nub
 
 from easybuild.framework.easyconfig.tools import process_easyconfig
 from easybuild.tools import config
-from easybuild.tools.filetools import mkdir, read_file, write_file
-from easybuild.tools.modules import curr_module_paths
+from easybuild.tools.filetools import mkdir, read_file, remove_file, write_file
 from easybuild.tools.module_generator import ModuleGeneratorLua, ModuleGeneratorTcl, dependencies_for
 from easybuild.tools.module_naming_scheme.utilities import is_valid_module_name
 from easybuild.framework.easyblock import EasyBlock
@@ -50,6 +48,7 @@ from easybuild.tools.modules import EnvironmentModulesC, Lmod
 from easybuild.tools.utilities import quote_str
 from test.framework.utilities import EnhancedTestCase, TestLoaderFiltered, find_full_path, init_config
 
+
 class ModuleGeneratorTest(EnhancedTestCase):
     """Tests for module_generator module."""
 
@@ -95,6 +94,7 @@ class ModuleGeneratorTest(EnhancedTestCase):
                 '',
                 "module-whatis {Description: %s}" % descr,
                 "module-whatis {Homepage: %s}" % homepage,
+                "module-whatis {URL: %s}" % homepage,
                 '',
                 "set root %s" % self.modgen.app.installdir,
                 '',
@@ -118,6 +118,7 @@ class ModuleGeneratorTest(EnhancedTestCase):
                 '',
                 "whatis([==[Description: %s]==])" % descr,
                 "whatis([==[Homepage: %s]==])" % homepage,
+                "whatis([==[URL: %s]==])" % homepage,
                 '',
                 'local root = "%s"' % self.modgen.app.installdir,
                 '',
@@ -234,6 +235,33 @@ class ModuleGeneratorTest(EnhancedTestCase):
         self.assertTrue(full_module_name in self.modtool.loaded_modules())
         self.modtool.purge()
 
+    def test_is_loaded(self):
+        """Test is_loaded method."""
+        if self.MODULE_GENERATOR_CLASS == ModuleGeneratorTcl:
+            test_cases = [
+                # single module name as argument
+                ('foo', "is-loaded foo"),
+                ('Python/2.7.15-GCCcore-8.2.0', "is-loaded Python/2.7.15-GCCcore-8.2.0"),
+                ('%(mod_name)s', "is-loaded %(mod_name)s"),
+                # list of multiple module names as argument should result in list of is-loaded statements
+                (['foo'], ['is-loaded foo']),
+                (['foo/1.2.3', 'bar/4.5.6'], ['is-loaded foo/1.2.3', 'is-loaded bar/4.5.6']),
+                (['foo', 'bar', 'baz'], ['is-loaded foo', 'is-loaded bar', 'is-loaded baz']),
+            ]
+        else:
+            test_cases = [
+                # single module name as argument
+                ('foo', 'isloaded("foo")'),
+                ('Python/2.7.15-GCCcore-8.2.0', 'isloaded("Python/2.7.15-GCCcore-8.2.0")'),
+                ('%(mod_name)s', 'isloaded("%(mod_name)s")'),
+                # list of multiple module names as argument
+                (['foo'], ['isloaded("foo")']),
+                (['foo/1.2.3', 'bar/4.5.6'], ['isloaded("foo/1.2.3")', 'isloaded("bar/4.5.6")']),
+                (['foo', 'bar', 'baz'], ['isloaded("foo")', 'isloaded("bar")', 'isloaded("baz")']),
+            ]
+
+        for mod_names, expected in test_cases:
+            self.assertEqual(self.modgen.is_loaded(mod_names), expected)
 
     def test_load(self):
         """Test load part in generated module file."""
@@ -252,7 +280,9 @@ class ModuleGeneratorTest(EnhancedTestCase):
             # with recursive unloading: no if is-loaded guard
             expected = '\n'.join([
                 '',
-                "module load mod_name",
+                "if { [ module-info mode remove ] || ![ is-loaded mod_name ] } {",
+                "    module load mod_name",
+                "}",
                 '',
             ])
             self.assertEqual(expected, self.modgen.load_module("mod_name", recursive_unload=True))
@@ -279,7 +309,7 @@ class ModuleGeneratorTest(EnhancedTestCase):
             # default: guarded module load (which implies no recursive unloading)
             expected = '\n'.join([
                 '',
-                'if not isloaded("mod_name") then',
+                'if not ( isloaded("mod_name") ) then',
                 '    load("mod_name")',
                 'end',
                 '',
@@ -290,7 +320,7 @@ class ModuleGeneratorTest(EnhancedTestCase):
             # check
             expected = '\n'.join([
                 '',
-                'if not isloaded("mod_name") or mode() == "unload" then',
+                'if mode() == "unload" or not ( isloaded("mod_name") ) then',
                 '    load("mod_name")',
                 'end',
                 '',
@@ -316,6 +346,140 @@ class ModuleGeneratorTest(EnhancedTestCase):
                 init_config(build_options={'mod_depends_on': 'True'})
                 self.assertErrorRegex(EasyBuildError, expected, self.modgen.load_module, "mod_name")
 
+    def test_load_multi_deps(self):
+        """Test generated load statement when multi_deps is involved."""
+
+        # first check with typical two-version multi_deps
+        multi_dep_mods = ['Python/3.7.4', 'Python/2.7.16']
+        res = self.modgen.load_module('Python/3.7.4', multi_dep_mods=multi_dep_mods)
+
+        if self.MODULE_GENERATOR_CLASS == ModuleGeneratorTcl:
+            expected = '\n'.join([
+                '',
+                "if { ![ is-loaded Python/3.7.4 ] && ![ is-loaded Python/2.7.16 ] } {",
+                "    module load Python/3.7.4",
+                '}',
+                '',
+            ])
+        else:  # Lua syntax
+            expected = '\n'.join([
+                '',
+                'if not ( isloaded("Python/3.7.4") ) and not ( isloaded("Python/2.7.16") ) then',
+                '    load("Python/3.7.4")',
+                'end',
+                '',
+            ])
+        self.assertEqual(expected, res)
+
+        if self.modtool.supports_depends_on:
+            # two versions with depends_on
+            res = self.modgen.load_module('Python/3.7.4', multi_dep_mods=multi_dep_mods, depends_on=True)
+
+            if self.MODULE_GENERATOR_CLASS == ModuleGeneratorTcl:
+                expected = '\n'.join([
+                    '',
+                    "if { [ module-info mode remove ] || [ is-loaded Python/2.7.16 ] } {",
+                    "    depends-on Python",
+                    '} else {',
+                    "    depends-on Python/3.7.4",
+                    '}',
+                    '',
+                ])
+            else:  # Lua syntax
+                expected = '\n'.join([
+                    '',
+                    'if mode() == "unload" or isloaded("Python/2.7.16") then',
+                    '    depends_on("Python")',
+                    'else',
+                    '    depends_on("Python/3.7.4")',
+                    'end',
+                    '',
+                ])
+            self.assertEqual(expected, res)
+
+        # now test with more than two versions...
+        multi_dep_mods = ['foo/1.2.3', 'foo/2.3.4', 'foo/3.4.5', 'foo/4.5.6']
+        res = self.modgen.load_module('foo/1.2.3', multi_dep_mods=multi_dep_mods)
+
+        if self.MODULE_GENERATOR_CLASS == ModuleGeneratorTcl:
+            expected = '\n'.join([
+                '',
+                "if { ![ is-loaded foo/1.2.3 ] && ![ is-loaded foo/2.3.4 ] && " +
+                "![ is-loaded foo/3.4.5 ] && ![ is-loaded foo/4.5.6 ] } {",
+                "    module load foo/1.2.3",
+                '}',
+                '',
+            ])
+        else:  # Lua syntax
+            expected = '\n'.join([
+                '',
+                'if not ( isloaded("foo/1.2.3") ) and not ( isloaded("foo/2.3.4") ) and ' +
+                'not ( isloaded("foo/3.4.5") ) and not ( isloaded("foo/4.5.6") ) then',
+                '    load("foo/1.2.3")',
+                'end',
+                '',
+            ])
+        self.assertEqual(expected, res)
+
+        if self.modtool.supports_depends_on:
+            # more than two versions, with depends_on
+            res = self.modgen.load_module('foo/1.2.3', multi_dep_mods=multi_dep_mods, depends_on=True)
+
+            if self.MODULE_GENERATOR_CLASS == ModuleGeneratorTcl:
+                expected = '\n'.join([
+                    '',
+                    "if { [ module-info mode remove ] || [ is-loaded foo/2.3.4 ] || [ is-loaded foo/3.4.5 ] " +
+                    "|| [ is-loaded foo/4.5.6 ] } {",
+                    "    depends-on foo",
+                    "} else {",
+                    "    depends-on foo/1.2.3",
+                    '}',
+                    '',
+                ])
+            else:  # Lua syntax
+                expected = '\n'.join([
+                    '',
+                    'if mode() == "unload" or isloaded("foo/2.3.4") or isloaded("foo/3.4.5") or ' +
+                    'isloaded("foo/4.5.6") then',
+                    '    depends_on("foo")',
+                    'else',
+                    '    depends_on("foo/1.2.3")',
+                    'end',
+                    '',
+                ])
+            self.assertEqual(expected, res)
+
+        # what if we only list a single version?
+        # see https://github.com/easybuilders/easybuild-framework/issues/3080
+        res = self.modgen.load_module('one/1.0', multi_dep_mods=['one/1.0'])
+
+        if self.MODULE_GENERATOR_CLASS == ModuleGeneratorTcl:
+            expected = '\n'.join([
+                '',
+                "if { ![ is-loaded one/1.0 ] } {",
+                "    module load one/1.0",
+                '}',
+                '',
+            ])
+        else:  # Lua syntax
+            expected = '\n'.join([
+                '',
+                'if not ( isloaded("one/1.0") ) then',
+                '    load("one/1.0")',
+                'end',
+                '',
+            ])
+        self.assertEqual(expected, res)
+
+        if self.modtool.supports_depends_on:
+            res = self.modgen.load_module('one/1.0', multi_dep_mods=['one/1.0'], depends_on=True)
+
+            if self.MODULE_GENERATOR_CLASS == ModuleGeneratorTcl:
+                expected = '\ndepends-on one/1.0\n'
+            else:  # Lua syntax
+                expected = '\ndepends_on("one/1.0")\n'
+            self.assertEqual(expected, res)
+
     def test_modulerc(self):
         """Test modulerc method."""
         self.assertErrorRegex(EasyBuildError, "Incorrect module_version value type", self.modgen.modulerc, 'foo')
@@ -370,21 +534,57 @@ class ModuleGeneratorTest(EnhancedTestCase):
         self.assertEqual(len(res), 1)
         self.assertEqual(res[0]['mod_name'], 'test/1.2.3.4.5')
 
-        # overwriting existing .modulerc requires --force or --rebuild
-        error_msg = "Found existing .modulerc at .*, not overwriting without --force or --rebuild"
-        self.assertErrorRegex(EasyBuildError, error_msg, self.modgen.modulerc, mod_ver_spec, filepath=modulerc_path)
-
-        init_config(build_options={'force': True})
-        modulerc = self.modgen.modulerc(mod_ver_spec, filepath=modulerc_path)
-        self.assertEqual(modulerc, expected)
+        # if same symbolic version is added again, nothing changes
+        self.modgen.modulerc(mod_ver_spec, filepath=modulerc_path)
         self.assertEqual(read_file(modulerc_path), expected)
 
-        init_config(build_options={})
-        self.assertErrorRegex(EasyBuildError, error_msg, self.modgen.modulerc, mod_ver_spec, filepath=modulerc_path)
+        # adding another module version results in appending to existing .modulerc file
+        write_file(os.path.join(self.test_prefix, 'test', '4.5.6'), '#%Module')
+        mod_ver_spec = {'modname': 'test/4.5.6', 'sym_version': '4', 'version': '4.5.6'}
+        self.modgen.modulerc(mod_ver_spec, filepath=modulerc_path)
+
+        if self.modtool.__class__ == EnvironmentModulesC:
+            expected += '\n'.join([
+                '',
+                'if {"test/4" eq [module-info version test/4]} {',
+                '    module-version test/4.5.6 4',
+                '}',
+            ])
+        elif self.MODULE_GENERATOR_CLASS == ModuleGeneratorLua:
+            if isinstance(self.modtool, Lmod) and LooseVersion(self.modtool.version) >= LooseVersion('7.8'):
+                expected += '\nmodule_version("test/4.5.6", "4")'
+            else:
+                expected += "\nmodule-version test/4.5.6 4"
+        else:
+            expected += "\nmodule-version test/4.5.6 4"
+
+        self.assertEqual(read_file(modulerc_path), expected)
+
+        # adding same symbolic version again doesn't cause trouble or changes...
+        self.modgen.modulerc(mod_ver_spec, filepath=modulerc_path)
+        self.assertEqual(read_file(modulerc_path), expected)
+
+        # starting from scratch yields expected results (only last symbolic version present)
+        remove_file(modulerc_path)
+        self.modgen.modulerc(mod_ver_spec, filepath=modulerc_path)
+
+        expected = '\n'.join([
+            '#%Module',
+            "module-version test/4.5.6 4",
+        ])
+
+        # two exceptions: EnvironmentModulesC, or Lmod 7.8 (or newer) and Lua syntax
+        if self.modtool.__class__ == EnvironmentModulesC:
+            expected = '\n'.join([
+                '#%Module',
+                'if {"test/4" eq [module-info version test/4]} {',
+                '    module-version test/4.5.6 4',
+                '}',
+            ])
+        elif self.MODULE_GENERATOR_CLASS == ModuleGeneratorLua:
+            if isinstance(self.modtool, Lmod) and LooseVersion(self.modtool.version) >= LooseVersion('7.8'):
+                expected = 'module_version("test/4.5.6", "4")'
 
-        init_config(build_options={'rebuild': True})
-        modulerc = self.modgen.modulerc(mod_ver_spec, filepath=modulerc_path)
-        self.assertEqual(modulerc, expected)
         self.assertEqual(read_file(modulerc_path), expected)
 
     def test_unload(self):
@@ -491,10 +691,37 @@ class ModuleGeneratorTest(EnhancedTestCase):
             res = self.modgen.append_paths('key', ['1234@example.com'], expand_relpaths=False)
             self.assertEqual('append_path("key", "1234@example.com")\n', res)
 
-        self.assertErrorRegex(EasyBuildError, "Absolute path %s/foo passed to update_paths " \
+        self.assertErrorRegex(EasyBuildError, "Absolute path %s/foo passed to update_paths "
                                               "which only expects relative paths." % self.modgen.app.installdir,
                               self.modgen.append_paths, "key2", ["bar", "%s/foo" % self.modgen.app.installdir])
 
+    def test_module_extensions(self):
+        """test the extensions() for extensions"""
+        # not supported for Tcl modules
+        if self.MODULE_GENERATOR_CLASS == ModuleGeneratorTcl:
+            return
+
+        # currently requires opt-in via --module-extensions
+        init_config(build_options={'module_extensions': True})
+
+        test_dir = os.path.abspath(os.path.dirname(__file__))
+        os.environ['MODULEPATH'] = os.path.join(test_dir, 'modules')
+        test_ec = os.path.join(test_dir, 'easyconfigs', 'test_ecs', 't', 'toy', 'toy-0.0-gompi-2018a-test.eb')
+
+        ec = EasyConfig(test_ec)
+        eb = EasyBlock(ec)
+        modgen = self.MODULE_GENERATOR_CLASS(eb)
+        desc = modgen.get_description()
+
+        patterns = [
+            r'^if convertToCanonical\(LmodVersion\(\)\) >= convertToCanonical\("8\.2\.8"\) then\n' +
+            r'\s*extensions\("bar/0.0,barbar/0.0,ls,toy/0.0"\)\nend$',
+        ]
+
+        for pattern in patterns:
+            regex = re.compile(pattern, re.M)
+            self.assertTrue(regex.search(desc), "Pattern '%s' found in: %s" % (regex.pattern, desc))
+
     def test_prepend_paths(self):
         """Test generating prepend-paths statements."""
         # test prepend_paths
@@ -539,7 +766,7 @@ class ModuleGeneratorTest(EnhancedTestCase):
             res = self.modgen.prepend_paths('key', ['1234@example.com'], expand_relpaths=False)
             self.assertEqual('prepend_path("key", "1234@example.com")\n', res)
 
-        self.assertErrorRegex(EasyBuildError, "Absolute path %s/foo passed to update_paths " \
+        self.assertErrorRegex(EasyBuildError, "Absolute path %s/foo passed to update_paths "
                                               "which only expects relative paths." % self.modgen.app.installdir,
                               self.modgen.prepend_paths, "key2", ["bar", "%s/foo" % self.modgen.app.installdir])
 
@@ -600,7 +827,7 @@ class ModuleGeneratorTest(EnhancedTestCase):
         # test set_environment
         if self.MODULE_GENERATOR_CLASS == ModuleGeneratorTcl:
             self.assertEqual('setenv\tkey\t\t"value"\n', self.modgen.set_environment("key", "value"))
-            self.assertEqual("setenv\tkey\t\t'va\"lue'\n", self.modgen.set_environment("key", 'va"lue'))
+            self.assertEqual('setenv\tkey\t\t"va\\"lue"\n', self.modgen.set_environment("key", 'va"lue'))
             self.assertEqual('setenv\tkey\t\t"va\'lue"\n', self.modgen.set_environment("key", "va'lue"))
             self.assertEqual('setenv\tkey\t\t"""va"l\'ue"""\n', self.modgen.set_environment("key", """va"l'ue"""))
         else:
@@ -620,12 +847,53 @@ class ModuleGeneratorTest(EnhancedTestCase):
         if self.MODULE_GENERATOR_CLASS == ModuleGeneratorTcl:
             # test set_alias
             self.assertEqual('set-alias\tkey\t\t"value"\n', self.modgen.set_alias("key", "value"))
-            self.assertEqual("set-alias\tkey\t\t'va\"lue'\n", self.modgen.set_alias("key", 'va"lue'))
+            self.assertEqual('set-alias\tkey\t\t"va\\"lue"\n', self.modgen.set_alias("key", 'va"lue'))
             self.assertEqual('set-alias\tkey\t\t"va\'lue"\n', self.modgen.set_alias("key", "va'lue"))
             self.assertEqual('set-alias\tkey\t\t"""va"l\'ue"""\n', self.modgen.set_alias("key", """va"l'ue"""))
         else:
             self.assertEqual('set_alias("key", "value")\n', self.modgen.set_alias("key", "value"))
 
+    def test_tcl_quoting(self):
+        """
+        Test escaping of double quotes when using Tcl modules
+        """
+
+        # note: this is for Tcl syntax only, skipping when it is not the case
+        if self.MODULE_GENERATOR_CLASS == ModuleGeneratorLua:
+            return
+
+        # creating base path
+        base_path = os.path.join(self.test_prefix, 'all')
+        mkdir(base_path)
+
+        # creating package module
+        module_name = 'tcl_quoting_mod'
+        modules_base_path = os.path.join(base_path, module_name)
+        mkdir(modules_base_path)
+
+        # creating module that sets envvar with quotation marks
+        txt = self.modgen.MODULE_SHEBANG
+        if txt:
+            txt += '\n'
+        txt += self.modgen.get_description()
+        test_envvar = 'TEST_FLAGS'
+        test_flags = '-Xflags1="foo bar" -Xflags2="more flags" '
+        txt += self.modgen.set_environment(test_envvar, test_flags)
+
+        version_one = '1.0'
+        version_one_path = os.path.join(modules_base_path, version_one + self.modgen.MODULE_FILE_EXTENSION)
+        write_file(version_one_path, txt)
+
+        # using base_path to possible module load
+        self.modtool.use(base_path)
+
+        self.modtool.load([module_name])
+        full_module_name = module_name + '/' + version_one
+
+        self.assertTrue(full_module_name in self.modtool.loaded_modules())
+        self.assertEqual(os.getenv(test_envvar), test_flags)
+        self.modtool.purge()
+
     def test_conditional_statement(self):
         """Test formatting of conditional statements."""
         if self.MODULE_GENERATOR_CLASS == ModuleGeneratorTcl:
@@ -687,7 +955,7 @@ class ModuleGeneratorTest(EnhancedTestCase):
 
             neg_cond = self.modgen.conditional_statement(cond, load, negative=True)
             expected = '\n'.join([
-                'if not isloaded("foo") then',
+                'if not ( isloaded("foo") ) then',
                 '    load("bar")',
                 'end',
                 '',
@@ -840,13 +1108,13 @@ class ModuleGeneratorTest(EnhancedTestCase):
         init_config(build_options=build_options)
         # note: these checksums will change if another easyconfig parameter is added
         ec2mod_map = {
-            'GCC-4.6.3.eb': 'GCC/5e4c8db5c005867c2aa9c1019500ed2cb1b4cf29',
-            'gzip-1.4.eb': 'gzip/53d5c13e85cb6945bd43a58d1c8d4a4c02f3462d',
+            'GCC-4.6.3.eb': 'GCC/355ab0c0b66cedfd6e87695ef152a0ebe45b8b28',
+            'gzip-1.4.eb': 'gzip/c2e522ded75b05c2b2074042fc39b5562b9929c3',
             'gzip-1.4-GCC-4.6.3.eb': 'gzip/585eba598f33c64ef01c6fa47af0fc37f3751311',
             'gzip-1.5-foss-2018a.eb': 'gzip/65dc39f92bf634667c478c50e43f0cda96b093a9',
             'gzip-1.5-intel-2018a.eb': 'gzip/0a4725f4720103eff8ffdadf8ffb187b988fb805',
-            'toy-0.0.eb': 'toy/cb0859b7b15723c826cd8504e5fde2573ab7b687',
-            'toy-0.0-multiple.eb': 'toy/cb0859b7b15723c826cd8504e5fde2573ab7b687',
+            'toy-0.0.eb': 'toy/d3cd467f89ab0bce1f2bcd553315524a3a5c8b34',
+            'toy-0.0-multiple.eb': 'toy/d3cd467f89ab0bce1f2bcd553315524a3a5c8b34',
         }
         test_mns()
 
@@ -858,7 +1126,7 @@ class ModuleGeneratorTest(EnhancedTestCase):
                 'name': 'GCC',
                 'version': '4.6.3',
                 'versionsuffix': '',
-                'toolchain': {'name': 'dummy', 'version': 'dummy'},
+                'toolchain': {'name': 'system', 'version': 'system'},
                 'hidden': False,
             }),
             ('gzip-1.5-foss-2018a.eb', {
@@ -872,7 +1140,7 @@ class ModuleGeneratorTest(EnhancedTestCase):
                 'name': 'toy',
                 'version': '0.0',
                 'versionsuffix': '-multiple',
-                'toolchain': {'name': 'dummy', 'version': 'dummy'},
+                'toolchain': {'name': 'system', 'version': 'system'},
                 'hidden': False,
             }),
         ]:
@@ -881,7 +1149,7 @@ class ModuleGeneratorTest(EnhancedTestCase):
 
         ec = EasyConfig(os.path.join(ecs_dir, 'g', 'gzip', 'gzip-1.5-foss-2018a.eb'), hidden=True)
         self.assertEqual(ec.full_mod_name, ec2mod_map['gzip-1.5-foss-2018a.eb'])
-        self.assertEqual(ec.toolchain.det_short_module_name(), 'foss/0c5d3fad1328e36c93258863f21234f4ff3f7a3f')
+        self.assertEqual(ec.toolchain.det_short_module_name(), 'foss/e69469ac250145c9e814e5dde93f5fde6d80375d')
 
         # restore default module naming scheme, and retest
         os.environ['EASYBUILD_MODULE_NAMING_SCHEME'] = self.orig_module_naming_scheme
@@ -949,7 +1217,7 @@ class ModuleGeneratorTest(EnhancedTestCase):
 
         def test_ec(ecfile, short_modname, mod_subdir, modpath_exts, user_modpath_exts, init_modpaths):
             """Test whether active module naming scheme returns expected values."""
-            ec = EasyConfig(glob.glob(os.path.join(ecs_dir, '*','*', ecfile))[0])
+            ec = EasyConfig(glob.glob(os.path.join(ecs_dir, '*', '*', ecfile))[0])
             self.assertEqual(ActiveMNS().det_full_module_name(ec), os.path.join(mod_subdir, short_modname))
             self.assertEqual(ActiveMNS().det_short_module_name(ec), short_modname)
             self.assertEqual(ActiveMNS().det_module_subdir(ec), mod_subdir)
@@ -978,6 +1246,10 @@ class ModuleGeneratorTest(EnhancedTestCase):
                                                  ['Compiler/intel/%s' % iccver], ['Core']),
             'ifort-2016.1.150.eb': ('ifort/2016.1.150', 'Core', ['Compiler/intel/2016.1.150'],
                                     ['Compiler/intel/2016.1.150'], ['Core']),
+            'iccifort-2019.4.243.eb': ('iccifort/2019.4.243', 'Core', ['Compiler/intel/2019.4.243'],
+                                       ['Compiler/intel/2019.4.243'], ['Core']),
+            'imkl-2019.4.243-iimpi-2019.08.eb': ('imkl/2019.4.243',
+                                                 'MPI/intel/2019.4.243/impi/2019.4.243', [], [], ['Core']),
             'CUDA-9.1.85-GCC-6.4.0-2.28.eb': ('CUDA/9.1.85', 'Compiler/GCC/6.4.0-2.28',
                                               ['Compiler/GCC-CUDA/6.4.0-2.28-9.1.85'],
                                               ['Compiler/GCC-CUDA/6.4.0-2.28-9.1.85'], ['Core']),
@@ -988,14 +1260,19 @@ class ModuleGeneratorTest(EnhancedTestCase):
                                                                   ['Compiler/intel-CUDA/%s-5.5.22' % iccver],
                                                                   ['Compiler/intel-CUDA/%s-5.5.22' % iccver],
                                                                   ['Core']),
+            'CUDA-10.1.243-iccifort-2019.4.243.eb': ('CUDA/10.1.243',
+                                                     'Compiler/intel/2019.4.243',
+                                                     ['Compiler/intel-CUDA/2019.4.243-10.1.243'],
+                                                     ['Compiler/intel-CUDA/2019.4.243-10.1.243'],
+                                                     ['Core']),
             impi_ec: ('impi/5.1.2.150', 'Compiler/intel/%s' % iccver, ['MPI/intel/%s/impi/5.1.2.150' % iccver],
                       ['MPI/intel/%s/impi/5.1.2.150' % iccver], ['Core']),
             imkl_ec: ('imkl/11.3.1.150', 'MPI/intel/%s/impi/5.1.2.150' % iccver, [],
                       [], ['Core']),
-            'impi-5.1.2.150-iccifortcuda-test.eb': ('impi/5.1.2.150', 'Compiler/intel-CUDA/%s-5.5.22' % iccver,
-                                                    ['MPI/intel-CUDA/%s-5.5.22/impi/5.1.2.150' % iccver],
-                                                    ['MPI/intel-CUDA/%s-5.5.22/impi/5.1.2.150' % iccver],
-                                                    ['Core']),
+            'impi-5.1.2.150-iccifortcuda-2016.1.150.eb': ('impi/5.1.2.150', 'Compiler/intel-CUDA/%s-5.5.22' % iccver,
+                                                          ['MPI/intel-CUDA/%s-5.5.22/impi/5.1.2.150' % iccver],
+                                                          ['MPI/intel-CUDA/%s-5.5.22/impi/5.1.2.150' % iccver],
+                                                          ['Core']),
         }
         for ecfile, mns_vals in test_ecs.items():
             test_ec(ecfile, *mns_vals)
@@ -1083,6 +1360,52 @@ class ModuleGeneratorTest(EnhancedTestCase):
         # only with depth=0, only direct dependencies are returned
         self.assertEqual(dependencies_for('foss/2018a', self.modtool, depth=0), expected[:-2])
 
+        # Lmod 7.6+ is required to use depends-on
+        if self.modtool.supports_depends_on:
+            # also test on module file that includes depends_on statements
+            test_modfile = os.path.join(self.test_prefix, 'test', '1.2.3')
+
+            if self.MODULE_GENERATOR_CLASS == ModuleGeneratorLua:
+                test_modtxt = '\n'.join([
+                    'depends_on("GCC/6.4.0-2.28")',
+                    'depends_on("OpenMPI/2.1.2-GCC-6.4.0-2.28")',
+                ])
+                test_modfile += '.lua'
+            else:
+                test_modtxt = '\n'.join([
+                    '#%Module',
+                    "depends-on GCC/6.4.0-2.28",
+                    "depends-on OpenMPI/2.1.2-GCC-6.4.0-2.28",
+                ])
+
+            write_file(test_modfile, test_modtxt)
+
+            self.modtool.use(self.test_prefix)
+
+            expected = [
+                'GCC/6.4.0-2.28',
+                'OpenMPI/2.1.2-GCC-6.4.0-2.28',
+                'hwloc/1.11.8-GCC-6.4.0-2.28',  # recursive dep, via OpenMPI
+            ]
+            self.assertEqual(dependencies_for('test/1.2.3', self.modtool), expected)
+
+    def test_det_installdir(self):
+        """Test det_installdir method."""
+
+        # first create a module file we can test with
+        modtxt = self.modgen.MODULE_SHEBANG
+        if modtxt:
+            modtxt += '\n'
+
+        modtxt += self.modgen.get_description()
+
+        test_modfile = os.path.join(self.test_prefix, 'test' + self.modgen.MODULE_FILE_EXTENSION)
+        write_file(test_modfile, modtxt)
+
+        expected = self.modgen.app.installdir
+
+        self.assertEqual(self.modgen.det_installdir(test_modfile), expected)
+
 
 class TclModuleGeneratorTest(ModuleGeneratorTest):
     """Test for module_generator module for Tcl syntax."""
@@ -1103,6 +1426,5 @@ def suite():
 
 
 if __name__ == '__main__':
-    #logToScreen(enable=True)
-    #setLogLevelDebug()
-    TextTestRunner(verbosity=1).run(suite())
+    res = TextTestRunner(verbosity=1).run(suite())
+    sys.exit(len(res.failures))
