@@ -1,5 +1,5 @@
 ##
-# Copyright 2012-2019 Ghent University
+# Copyright 2012-2020 Ghent University
 #
 # This file is part of EasyBuild,
 # originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
@@ -74,9 +74,9 @@ class RepositoryTest(EnhancedTestCase):
         """Test using GitRepository."""
         # only run this test if git Python module is available
         try:
-            from git import GitCommandError
+            from git import GitCommandError  # noqa
         except ImportError:
-            print "(skipping GitRepository test)"
+            print("(skipping GitRepository test)")
             return
 
         test_repo_url = 'https://github.com/hpcugent/testrepository'
@@ -88,8 +88,8 @@ class RepositoryTest(EnhancedTestCase):
             self.assertEqual(os.path.basename(repo.wc), 'testrepository')
             self.assertTrue(os.path.exists(os.path.join(repo.wc, 'README.md')))
             shutil.rmtree(repo.wc)
-        except EasyBuildError, err:
-            print "ignoring failed subtest in test_gitrepo, testing offline?"
+        except EasyBuildError as err:
+            print("ignoring failed subtest in test_gitrepo, testing offline?")
             self.assertTrue(re.search("pull in working copy .* went wrong", str(err)))
 
         # filepath
@@ -116,9 +116,9 @@ class RepositoryTest(EnhancedTestCase):
         """Test using SvnRepository."""
         # only run this test if pysvn Python module is available
         try:
-            from pysvn import ClientError
+            from pysvn import ClientError  # noqa
         except ImportError:
-            print "(skipping SvnRepository test)"
+            print("(skipping SvnRepository test)")
             return
 
         # GitHub also supports SVN
@@ -137,9 +137,9 @@ class RepositoryTest(EnhancedTestCase):
         """Test using HgRepository."""
         # only run this test if pysvn Python module is available
         try:
-            import hglib
+            import hglib  # noqa
         except ImportError:
-            print "(skipping HgRepository test)"
+            print("(skipping HgRepository test)")
             return
 
         # GitHub also supports SVN
@@ -193,12 +193,14 @@ class RepositoryTest(EnhancedTestCase):
             path = repo.add_easyconfig(toy_yeb_file, 'test', '1.0', {'time': 1.23}, None)
             check_ec(path, [{'time': 1.23}])
 
-            path = repo.add_easyconfig(toy_yeb_file, 'test', '1.0', {'time': 1.23, 'size': 123}, [{'time': 0.9, 'size': 2}])
-            check_ec(path, [{'time': 0.9, 'size': 2}, {'time': 1.23, 'size': 123}])
+            stats1 = {'time': 1.23, 'size': 123}
+            stats2 = [{'time': 0.9, 'size': 2}]
+            path = repo.add_easyconfig(toy_yeb_file, 'test', '1.0', stats1, stats2)
+            check_ec(path, stats2 + [stats1])
 
             easybuild.tools.build_log.EXPERIMENTAL = orig_experimental
         else:
-            print "Skipping .yeb part of test_add_easyconfig (no PyYAML available)"
+            print("Skipping .yeb part of test_add_easyconfig (no PyYAML available)")
 
     def tearDown(self):
         """Clean up after test."""
@@ -206,10 +208,12 @@ class RepositoryTest(EnhancedTestCase):
 
         shutil.rmtree(self.path, True)
 
+
 def suite():
     """ returns all the testcases in this module """
     return TestLoaderFiltered().loadTestsFromTestCase(RepositoryTest, sys.argv[1:])
 
 
 if __name__ == '__main__':
-    TextTestRunner(verbosity=1).run(suite())
+    res = TextTestRunner(verbosity=1).run(suite())
+    sys.exit(len(res.failures))
