@@ -1,5 +1,5 @@
 ##
-# Copyright 2012-2019 Ghent University
+# Copyright 2012-2020 Ghent University
 #
 # This file is part of EasyBuild,
 # originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
@@ -42,11 +42,14 @@ from test.framework.utilities import EnhancedTestCase, TestLoaderFiltered, find_
 import easybuild.tools.modules as modules
 import easybuild.tools.toolchain.compiler
 from easybuild.framework.easyconfig.easyconfig import EasyConfig, ActiveMNS
+from easybuild.toolchains.system import SystemToolchain
 from easybuild.tools import systemtools as st
 from easybuild.tools.build_log import EasyBuildError
 from easybuild.tools.environment import setvar
 from easybuild.tools.filetools import adjust_permissions, copy_dir, find_eb_script, mkdir, read_file, write_file, which
+from easybuild.tools.py2vs3 import string_type
 from easybuild.tools.run import run_cmd
+from easybuild.tools.toolchain.toolchain import env_vars_external_module
 from easybuild.tools.toolchain.utilities import get_toolchain, search_toolchain
 
 easybuild.tools.toolchain.compiler.systemtools.get_compiler_family = lambda: st.POWER
@@ -63,6 +66,8 @@ class ToolchainTest(EnhancedTestCase):
         self.orig_get_cpu_model = st.get_cpu_model
         self.orig_get_cpu_vendor = st.get_cpu_vendor
 
+        init_config(build_options={'silent': True})
+
     def tearDown(self):
         """Cleanup after toolchain test."""
         st.get_cpu_architecture = self.orig_get_cpu_architecture
@@ -92,23 +97,72 @@ class ToolchainTest(EnhancedTestCase):
         self.assertEqual(tc, None)
         self.assertTrue(len(all_tcs) > 0)  # list of available toolchains
 
+    def test_system_toolchain(self):
+        """Test for system toolchain."""
+        for ver in ['system', '']:
+            tc = self.get_toolchain('system', version=ver)
+            self.assertTrue(isinstance(tc, SystemToolchain))
+
     def test_foss_toolchain(self):
         """Test for foss toolchain."""
         self.get_toolchain("foss", version="2018a")
 
-    def test_get_variable_dummy_toolchain(self):
-        """Test get_variable on dummy toolchain"""
-        tc = self.get_toolchain('dummy', version='dummy')
-        tc.prepare()
-        self.assertEqual(tc.get_variable('CC'), '')
-        self.assertEqual(tc.get_variable('CXX', typ=str), '')
-        self.assertEqual(tc.get_variable('CFLAGS', typ=list), [])
+    def test_get_variable_system_toolchain(self):
+        """Test get_variable on system/dummy toolchain"""
 
-        tc = self.get_toolchain('dummy', version='')
-        tc.prepare()
-        self.assertEqual(tc.get_variable('CC'), '')
-        self.assertEqual(tc.get_variable('CXX', typ=str), '')
-        self.assertEqual(tc.get_variable('CFLAGS', typ=list), [])
+        # system toolchain version doesn't really matter, but fine...
+        for ver in ['system', '']:
+            tc = self.get_toolchain('system', version=ver)
+            tc.prepare()
+            self.assertEqual(tc.get_variable('CC'), '')
+            self.assertEqual(tc.get_variable('CXX', typ=str), '')
+            self.assertEqual(tc.get_variable('CFLAGS', typ=list), [])
+
+        # dummy toolchain is deprecated, so we need to allow for it (and catch the warnings that get printed)
+        self.allow_deprecated_behaviour()
+
+        for ver in ['dummy', '']:
+            self.mock_stderr(True)
+            tc = self.get_toolchain('dummy', version=ver)
+            self.mock_stderr(False)
+            tc.prepare()
+            self.assertEqual(tc.get_variable('CC'), '')
+            self.assertEqual(tc.get_variable('CXX', typ=str), '')
+            self.assertEqual(tc.get_variable('CFLAGS', typ=list), [])
+
+    def test_is_system_toolchain(self):
+        """Test is_system_toolchain method."""
+
+        init_config()
+
+        for ver in ['system', '']:
+            tc = self.get_toolchain('system', version=ver)
+            self.assertTrue(tc.is_system_toolchain())
+
+        tc = self.get_toolchain('foss', version='2018a')
+        self.assertFalse(tc.is_system_toolchain())
+
+        self.setup_sandbox_for_intel_fftw(self.test_prefix)
+        self.modtool.prepend_module_path(self.test_prefix)
+        tc = self.get_toolchain('intel', version='2018a')
+        self.assertFalse(tc.is_system_toolchain())
+
+        # using dummy toolchain is deprecated, so to test for that we need to explicitely allow using deprecated stuff
+        error_pattern = "Use of 'dummy' toolchain is deprecated"
+        for ver in ['dummy', '']:
+            self.assertErrorRegex(EasyBuildError, error_pattern, self.get_toolchain, 'dummy', version=ver)
+
+        dummy_depr_warning = "WARNING: Deprecated functionality, will no longer work in v5.0: Use of 'dummy' toolchain"
+
+        self.allow_deprecated_behaviour()
+
+        for ver in ['dummy', '']:
+            self.mock_stderr(True)
+            tc = self.get_toolchain('dummy', version=ver)
+            stderr = self.get_stderr()
+            self.mock_stderr(False)
+            self.assertTrue(tc.is_system_toolchain())
+            self.assertTrue(dummy_depr_warning in stderr, "Found '%s' in: %s" % (dummy_depr_warning, stderr))
 
     def test_get_variable_compilers(self):
         """Test get_variable function to obtain compiler variables."""
@@ -132,11 +186,8 @@ class ToolchainTest(EnhancedTestCase):
         self.assertEqual(tc.get_variable('OMPI_F77'), 'gfortran')
         self.assertEqual(tc.get_variable('OMPI_FC'), 'gfortran')
 
-    def test_get_variable_mpi_compilers(self):
-        """Test get_variable function to obtain compiler variables."""
-        tc = self.get_toolchain('foss', version='2018a')
-        tc.set_options({'usempi': True})
-        tc.prepare()
+    def check_vars_foss_usempi(self, tc):
+        """Utility function to check compiler variables for foss toolchain with usempi enabled."""
 
         self.assertEqual(tc.get_variable('CC'), 'mpicc')
         self.assertEqual(tc.get_variable('CXX'), 'mpicxx')
@@ -155,6 +206,50 @@ class ToolchainTest(EnhancedTestCase):
         self.assertEqual(tc.get_variable('OMPI_F77'), 'gfortran')
         self.assertEqual(tc.get_variable('OMPI_FC'), 'gfortran')
 
+    def test_get_variable_mpi_compilers(self):
+        """Test get_variable function to obtain compiler variables."""
+        tc = self.get_toolchain('foss', version='2018a')
+        tc.set_options({'usempi': True})
+        tc.prepare()
+
+        self.check_vars_foss_usempi(tc)
+
+    def test_prepare_iterate(self):
+        """Test preparing of toolchain in iterative context."""
+        tc = self.get_toolchain('foss', version='2018a')
+        tc.set_options({'usempi': True})
+
+        tc.prepare()
+        self.check_vars_foss_usempi(tc)
+
+        # without a reset, the value is wrong...
+        tc.prepare()
+        self.assertFalse(tc.get_variable('MPICC') == 'mpicc')
+
+        tc.reset()
+        tc.prepare()
+        self.check_vars_foss_usempi(tc)
+
+    def test_cray_reset(self):
+        """Test toolchain preparation after reset for Cray* toolchain."""
+        # cfr. https://github.com/easybuilders/easybuild-framework/issues/2911
+        init_config(build_options={'optarch': 'test', 'silent': True})
+
+        for tcname in ['CrayGNU', 'CrayCCE', 'CrayIntel']:
+            tc = self.get_toolchain(tcname, version='2015.06-XC')
+            tc.set_options({'dynamic': True})
+            tc.prepare()
+            self.assertEqual(os.environ.get('LIBBLAS'), '')
+            tc.reset()
+            tc.prepare()
+            self.assertEqual(os.environ.get('LIBBLAS'), '')
+            tc.reset()
+            tc.prepare()
+            self.assertEqual(os.environ.get('LIBBLAS'), '')
+            tc.reset()
+            tc.prepare()
+            self.assertEqual(os.environ.get('LIBBLAS'), '')
+
     def test_get_variable_seq_compilers(self):
         """Test get_variable function to obtain compiler variables."""
         tc = self.get_toolchain('foss', version='2018a')
@@ -175,7 +270,7 @@ class ToolchainTest(EnhancedTestCase):
         ldflags = tc.get_variable('LDFLAGS', typ=list)
         self.assertTrue(isinstance(ldflags, list))
         if len(ldflags) > 0:
-            self.assertTrue(isinstance(ldflags[0], basestring))
+            self.assertTrue(isinstance(ldflags[0], string_type))
 
     def test_validate_pass_by_value(self):
         """
@@ -312,8 +407,7 @@ class ToolchainTest(EnhancedTestCase):
         """Test whether overriding the optarch flag works."""
         flag_vars = ['CFLAGS', 'CXXFLAGS', 'FCFLAGS', 'FFLAGS', 'F90FLAGS']
         for optarch_var in ['march=lovelylovelysandybridge', None]:
-            build_options = {'optarch': optarch_var}
-            init_config(build_options=build_options)
+            init_config(build_options={'optarch': optarch_var, 'silent': True})
             for enable in [True, False]:
                 tc = self.get_toolchain('foss', version='2018a')
                 tc.set_options({'optarch': enable})
@@ -337,8 +431,7 @@ class ToolchainTest(EnhancedTestCase):
         """Test whether --optarch=GENERIC works as intended."""
         for generic in [False, True]:
             if generic:
-                build_options = {'optarch': 'GENERIC'}
-                init_config(build_options=build_options)
+                init_config(build_options={'optarch': 'GENERIC', 'silent': True})
             flag_vars = ['CFLAGS', 'CXXFLAGS', 'FCFLAGS', 'FFLAGS', 'F90FLAGS']
             tcs = {
                 'gompi': ('2018a', "-march=x86-64 -mtune=generic"),
@@ -410,8 +503,7 @@ class ToolchainTest(EnhancedTestCase):
             optarch_var['Intel'] = intel_flags
             optarch_var['GCC'] = gcc_flags
             optarch_var['GCCcore'] = gcccore_flags
-            build_options = {'optarch': optarch_var}
-            init_config(build_options=build_options)
+            init_config(build_options={'optarch': optarch_var, 'silent': True})
             tc = self.get_toolchain(toolchain, version=toolchain_ver)
             tc.set_options({'optarch': enable})
             tc.prepare()
@@ -429,7 +521,7 @@ class ToolchainTest(EnhancedTestCase):
 
             optarch_flags = tc.options.options_map['optarch']
 
-            self.assertEquals(flags, optarch_flags)
+            self.assertEqual(flags, optarch_flags)
 
             # Also check that it is correctly passed to xFLAGS, honoring 'enable'
             if flags == '':
@@ -602,6 +694,111 @@ class ToolchainTest(EnhancedTestCase):
         self.assertEqual(tc.blas_family(), 'IntelMKL')
         self.assertEqual(tc.lapack_family(), 'IntelMKL')
 
+    def test_fft_env_vars_foss(self):
+        """Test setting of $FFT* environment variables using foss toolchain."""
+        tc = self.get_toolchain('foss', version='2018a')
+        tc.prepare()
+
+        fft_static_libs = 'libfftw3.a'
+        self.assertEqual(tc.get_variable('FFT_STATIC_LIBS'), fft_static_libs)
+        self.assertEqual(tc.get_variable('FFTW_STATIC_LIBS'), fft_static_libs)
+
+        fft_static_libs_mt = 'libfftw3.a,libpthread.a'
+        self.assertEqual(tc.get_variable('FFT_STATIC_LIBS_MT'), fft_static_libs_mt)
+        self.assertEqual(tc.get_variable('FFTW_STATIC_LIBS_MT'), fft_static_libs_mt)
+
+        self.assertEqual(tc.get_variable('LIBFFT'), '-lfftw3')
+        self.assertEqual(tc.get_variable('LIBFFT_MT'), '-lfftw3 -lpthread')
+
+        tc = self.get_toolchain('foss', version='2018a')
+        tc.set_options({'openmp': True})
+        tc.prepare()
+
+        self.assertEqual(tc.get_variable('FFT_STATIC_LIBS'), fft_static_libs)
+        self.assertEqual(tc.get_variable('FFTW_STATIC_LIBS'), fft_static_libs)
+
+        self.assertEqual(tc.get_variable('FFT_STATIC_LIBS_MT'), 'libfftw3_omp.a,' + fft_static_libs_mt)
+        self.assertEqual(tc.get_variable('FFTW_STATIC_LIBS_MT'), 'libfftw3_omp.a,' + fft_static_libs_mt)
+
+        self.assertEqual(tc.get_variable('LIBFFT'), '-lfftw3')
+        self.assertEqual(tc.get_variable('LIBFFT_MT'), '-lfftw3_omp -lfftw3 -lpthread')
+
+        tc = self.get_toolchain('foss', version='2018a')
+        tc.set_options({'usempi': True})
+        tc.prepare()
+
+        fft_static_libs = 'libfftw3_mpi.a,libfftw3.a'
+        self.assertEqual(tc.get_variable('FFT_STATIC_LIBS'), fft_static_libs)
+        self.assertEqual(tc.get_variable('FFTW_STATIC_LIBS'), fft_static_libs)
+
+        self.assertEqual(tc.get_variable('FFT_STATIC_LIBS_MT'), fft_static_libs_mt)
+        self.assertEqual(tc.get_variable('FFTW_STATIC_LIBS_MT'), fft_static_libs_mt)
+
+        self.assertEqual(tc.get_variable('LIBFFT'), '-lfftw3_mpi -lfftw3')
+        self.assertEqual(tc.get_variable('LIBFFT_MT'), '-lfftw3 -lpthread')
+
+    def test_fft_env_vars_intel(self):
+        """Test setting of $FFT* environment variables using intel toolchain."""
+
+        self.setup_sandbox_for_intel_fftw(self.test_prefix)
+        self.modtool.prepend_module_path(self.test_prefix)
+
+        tc = self.get_toolchain('intel', version='2018a')
+        tc.prepare()
+
+        fft_static_libs = 'libfftw3xc_intel.a,libmkl_intel_lp64.a,libmkl_sequential.a,libmkl_core.a'
+        self.assertEqual(tc.get_variable('FFT_STATIC_LIBS'), fft_static_libs)
+        self.assertEqual(tc.get_variable('FFTW_STATIC_LIBS'), fft_static_libs)
+
+        fft_static_libs_mt = 'libfftw3xc_intel.a,libmkl_intel_lp64.a,libmkl_intel_thread.a,libmkl_core.a,'
+        fft_static_libs_mt += 'libiomp5.a,libpthread.a'
+        self.assertEqual(tc.get_variable('FFT_STATIC_LIBS_MT'), fft_static_libs_mt)
+        self.assertEqual(tc.get_variable('FFTW_STATIC_LIBS_MT'), fft_static_libs_mt)
+
+        libfft = "-Wl,-Bstatic -Wl,--start-group -lfftw3xc_intel -lmkl_intel_lp64 -lmkl_sequential -lmkl_core "
+        libfft += "-Wl,--end-group -Wl,-Bdynamic"
+        self.assertEqual(tc.get_variable('LIBFFT'), libfft)
+
+        libfft_mt = "-Wl,-Bstatic -Wl,--start-group -lfftw3xc_intel -lmkl_intel_lp64 -lmkl_intel_thread -lmkl_core "
+        libfft_mt += "-Wl,--end-group -Wl,-Bdynamic -liomp5 -lpthread"
+        self.assertEqual(tc.get_variable('LIBFFT_MT'), libfft_mt)
+
+        tc = self.get_toolchain('intel', version='2018a')
+        tc.set_options({'openmp': True})
+        tc.prepare()
+
+        self.assertEqual(tc.get_variable('FFT_STATIC_LIBS'), fft_static_libs)
+        self.assertEqual(tc.get_variable('FFTW_STATIC_LIBS'), fft_static_libs)
+
+        self.assertEqual(tc.get_variable('FFT_STATIC_LIBS_MT'), fft_static_libs_mt)
+        self.assertEqual(tc.get_variable('FFTW_STATIC_LIBS_MT'), fft_static_libs_mt)
+
+        self.assertEqual(tc.get_variable('LIBFFT'), libfft)
+        self.assertEqual(tc.get_variable('LIBFFT_MT'), libfft_mt)
+
+        tc = self.get_toolchain('intel', version='2018a')
+        tc.set_options({'usempi': True})
+        tc.prepare()
+
+        fft_static_libs = 'libfftw3xc_intel.a,libfftw3x_cdft_lp64.a,libmkl_cdft_core.a,libmkl_blacs_intelmpi_lp64.a,'
+        fft_static_libs += 'libmkl_intel_lp64.a,libmkl_sequential.a,libmkl_core.a'
+        self.assertEqual(tc.get_variable('FFT_STATIC_LIBS'), fft_static_libs)
+        self.assertEqual(tc.get_variable('FFTW_STATIC_LIBS'), fft_static_libs)
+
+        fft_static_libs_mt = 'libfftw3xc_intel.a,libfftw3x_cdft_lp64.a,libmkl_cdft_core.a,libmkl_blacs_intelmpi_lp64.a,'
+        fft_static_libs_mt += 'libmkl_intel_lp64.a,libmkl_intel_thread.a,libmkl_core.a,libiomp5.a,libpthread.a'
+        self.assertEqual(tc.get_variable('FFT_STATIC_LIBS_MT'), fft_static_libs_mt)
+        self.assertEqual(tc.get_variable('FFTW_STATIC_LIBS_MT'), fft_static_libs_mt)
+
+        libfft = '-Wl,-Bstatic -Wl,--start-group -lfftw3xc_intel -lfftw3x_cdft_lp64 -lmkl_cdft_core '
+        libfft += '-lmkl_blacs_intelmpi_lp64 -lmkl_intel_lp64 -lmkl_sequential -lmkl_core -Wl,--end-group -Wl,-Bdynamic'
+        self.assertEqual(tc.get_variable('LIBFFT'), libfft)
+
+        libfft_mt = '-Wl,-Bstatic -Wl,--start-group -lfftw3xc_intel -lfftw3x_cdft_lp64 -lmkl_cdft_core '
+        libfft_mt += '-lmkl_blacs_intelmpi_lp64 -lmkl_intel_lp64 -lmkl_intel_thread -lmkl_core -Wl,--end-group '
+        libfft_mt += '-Wl,-Bdynamic -liomp5 -lpthread'
+        self.assertEqual(tc.get_variable('LIBFFT_MT'), libfft_mt)
+
     def test_fosscuda(self):
         """Test whether fosscuda is handled properly."""
         tc = self.get_toolchain("fosscuda", version="2018a")
@@ -774,7 +971,7 @@ class ToolchainTest(EnhancedTestCase):
         self.assertTrue(mpi_cmd_for_re.match(tc.mpi_cmd_for('test', 4)))
 
         # test specifying custom template for MPI commands
-        init_config(build_options={'mpi_cmd_template': "mpiexec -np %(nr_ranks)s -- %(cmd)s"})
+        init_config(build_options={'mpi_cmd_template': "mpiexec -np %(nr_ranks)s -- %(cmd)s", 'silent': True})
         self.assertEqual(tc.mpi_cmd_for('test123', '7'), "mpiexec -np 7 -- test123")
 
     def test_prepare_deps(self):
@@ -929,11 +1126,76 @@ class ToolchainTest(EnhancedTestCase):
         self.assertEqual(os.environ['LIBFFT_MT'], libfft_mt_fosscuda)
         self.assertEqual(os.environ['LIBSCALAPACK'], libscalack_fosscuda)
 
+    def test_standalone_iccifort(self):
+        """Test whether standalone installation of iccifort matches the iccifort toolchain definition."""
+
+        tc = self.get_toolchain('iccifort', version='2018.1.163')
+        tc.prepare()
+        self.assertEqual(tc.toolchain_dep_mods, ['icc/2018.1.163', 'ifort/2018.1.163'])
+        self.modtool.purge()
+
+        for key in ['EBROOTICC', 'EBROOTIFORT', 'EBVERSIONICC', 'EBVERSIONIFORT']:
+            self.assertTrue(os.getenv(key) is None)
+
+        # install fake iccifort module with no dependencies
+        fake_iccifort = os.path.join(self.test_prefix, 'iccifort', '2018.1.163')
+        write_file(fake_iccifort, "#%Module")
+        self.modtool.use(self.test_prefix)
+
+        # toolchain verification fails because icc/ifort are not dependencies of iccifort modules,
+        # and corresponding environment variables are not set
+        error_pattern = "List of toolchain dependency modules and toolchain definition do not match"
+        self.assertErrorRegex(EasyBuildError, error_pattern, tc.prepare)
+        self.modtool.purge()
+
+        # make iccifort module set $EBROOT* and $EBVERSION* to pass toolchain verification
+        fake_iccifort_txt = '\n'.join([
+            "#%Module",
+            'setenv EBROOTICC "%s"' % self.test_prefix,
+            'setenv EBROOTIFORT "%s"' % self.test_prefix,
+            'setenv EBVERSIONICC "2018.1.163"',
+            'setenv EBVERSIONIFORT "2018.1.163"',
+        ])
+        write_file(fake_iccifort, fake_iccifort_txt)
+        # toolchain preparation (which includes verification) works fine now
+        tc.prepare()
+        # no dependencies found in iccifort module
+        self.assertEqual(tc.toolchain_dep_mods, [])
+
+    def test_standalone_iccifortcuda(self):
+        """Test whether standalone installation of iccifortcuda matches the iccifortcuda toolchain definition."""
+
+        tc = self.get_toolchain('iccifortcuda', version='2018b')
+        tc.prepare()
+        self.assertEqual(tc.toolchain_dep_mods, ['icc/2018.1.163', 'ifort/2018.1.163', 'CUDA/9.1.85'])
+        self.modtool.purge()
+
+        for key in ['EBROOTICC', 'EBROOTIFORT', 'EBVERSIONICC', 'EBVERSIONIFORT', 'EBROOTCUDA', 'EBVERSIONCUDA']:
+            self.assertTrue(os.getenv(key) is None)
+
+        # install fake iccifortcuda module with no dependencies
+        fake_iccifortcuda = os.path.join(self.test_prefix, 'iccifortcuda', '2018b')
+        write_file(fake_iccifortcuda, "#%Module")
+        self.modtool.use(self.test_prefix)
+
+        # toolchain verification fails because icc/ifort are not dependencies of iccifortcuda modules,
+        # and corresponding environment variables are not set
+        error_pattern = "List of toolchain dependency modules and toolchain definition do not match"
+        self.assertErrorRegex(EasyBuildError, error_pattern, tc.prepare)
+        self.modtool.purge()
+
+        # Verify that it works loading a module that contains a combined iccifort module
+        tc = self.get_toolchain('iccifortcuda', version='2019a')
+        # toolchain preparation (which includes verification) works fine now
+        tc.prepare()
+        # dependencies found in iccifortcuda module
+        self.assertEqual(tc.toolchain_dep_mods, ['iccifort/2019.5.281', 'CUDA/9.1.85'])
+
     def test_independence(self):
         """Test independency of toolchain instances."""
 
         # tweaking --optarch is required for Cray toolchains (craypre-<optarch> module must be available)
-        init_config(build_options={'optarch': 'test'})
+        init_config(build_options={'optarch': 'test', 'silent': True})
 
         tc_cflags = {
             'CrayCCE': "-O2 -homp -craype-verbose",
@@ -970,6 +1232,7 @@ class ToolchainTest(EnhancedTestCase):
         write_file(os.path.join(self.test_prefix, 'PGI', '14.9'), '#%Module\nsetenv EBVERSIONPGI 14.9')
         write_file(os.path.join(self.test_prefix, 'PGI', '14.10'), '#%Module\nsetenv EBVERSIONPGI 14.10')
         write_file(os.path.join(self.test_prefix, 'PGI', '16.3'), '#%Module\nsetenv EBVERSIONPGI 16.3')
+        write_file(os.path.join(self.test_prefix, 'PGI', '19.1'), '#%Module\nsetenv EBVERSIONPGI 19.1')
         self.modtool.prepend_module_path(self.test_prefix)
 
         tc = self.get_toolchain('PGI', version='14.9')
@@ -982,13 +1245,16 @@ class ToolchainTest(EnhancedTestCase):
         self.assertEqual(tc.get_variable('FC'), 'pgfortran')
         self.modtool.purge()
 
-        for pgi_ver in ['14.10', '16.3']:
+        for pgi_ver in ['14.10', '16.3', '19.1']:
             tc = self.get_toolchain('PGI', version=pgi_ver)
             tc.prepare()
 
             self.assertEqual(tc.get_variable('CC'), 'pgcc')
             self.assertEqual(tc.get_variable('CXX'), 'pgc++')
-            self.assertEqual(tc.get_variable('F77'), 'pgf77')
+            if pgi_ver == '19.1':
+                self.assertEqual(tc.get_variable('F77'), 'pgfortran')
+            else:
+                self.assertEqual(tc.get_variable('F77'), 'pgf77')
             self.assertEqual(tc.get_variable('F90'), 'pgf90')
             self.assertEqual(tc.get_variable('FC'), 'pgfortran')
 
@@ -1143,6 +1409,21 @@ class ToolchainTest(EnhancedTestCase):
         ]
         self.assertEqual(out.strip(), "CMD_ARGS=(%s)" % ' '.join(cmd_args))
 
+        # compiler command, -Wl,--enable-new-dtags should be replaced with -Wl,--disable-new-dtags
+        out, ec = run_cmd("%s gcc '' '%s' -Wl,--enable-new-dtags foo.c" % (script, rpath_inc), simple=False)
+        self.assertEqual(ec, 0)
+        cmd_args = [
+            "'-Wl,-rpath=%s/lib'" % self.test_prefix,
+            "'-Wl,-rpath=%s/lib64'" % self.test_prefix,
+            "'-Wl,-rpath=$ORIGIN'",
+            "'-Wl,-rpath=$ORIGIN/../lib'",
+            "'-Wl,-rpath=$ORIGIN/../lib64'",
+            "'-Wl,--disable-new-dtags'",
+            "'-Wl,--disable-new-dtags'",
+            "'foo.c'",
+        ]
+        self.assertEqual(out.strip(), "CMD_ARGS=(%s)" % ' '.join(cmd_args))
+
         # test passing no arguments
         out, ec = run_cmd("%s gcc '' '%s'" % (script, rpath_inc), simple=False)
         self.assertEqual(ec, 0)
@@ -1366,7 +1647,7 @@ class ToolchainTest(EnhancedTestCase):
         os.environ['PATH'] = '%s:%s' % (os.path.join(self.test_prefix, 'fake'), os.getenv('PATH', ''))
 
         # enable --rpath and prepare toolchain
-        init_config(build_options={'rpath': True, 'rpath_filter': ['/ba.*']})
+        init_config(build_options={'rpath': True, 'rpath_filter': ['/ba.*'], 'silent': True})
         tc = self.get_toolchain('gompi', version='2018a')
 
         # preparing RPATH wrappers requires --experimental, need to bypass that here
@@ -1467,6 +1748,9 @@ class ToolchainTest(EnhancedTestCase):
     def test_prepare_openmpi_tmpdir(self):
         """Test handling of long $TMPDIR path for OpenMPI 2.x"""
 
+        # this test relies on warnings being printed
+        init_config(build_options={'silent': False})
+
         def prep():
             """Helper function: create & prepare toolchain"""
             self.modtool.unload(['gompi', 'OpenMPI', 'hwloc', 'GCC'])
@@ -1501,13 +1785,14 @@ class ToolchainTest(EnhancedTestCase):
         os.environ['TMPDIR'] = long_tmpdir
         tc, stdout, stderr = prep()
         self.assertEqual(stdout, '')
-        regex = re.compile("^WARNING: Long \$TMPDIR .* problems with OpenMPI 2.x, using shorter path: /tmp/.{6}$")
+        # basename of tmpdir will be 6 chars in Python 2, 8 chars in Python 3
+        regex = re.compile(r"^WARNING: Long \$TMPDIR .* problems with OpenMPI 2.x, using shorter path: /tmp/.{6,8}$")
         self.assertTrue(regex.match(stderr), "Pattern '%s' found in: %s" % (regex.pattern, stderr))
 
         # new $TMPDIR should be /tmp/xxxxxx
         tmpdir = os.environ.get('TMPDIR')
         self.assertTrue(tmpdir.startswith('/tmp'))
-        self.assertEqual(len(tmpdir), 11)
+        self.assertTrue(len(tmpdir) in (11, 13))
 
         # also test cleanup method to ensure short $TMPDIR is cleaned up properly
         self.assertTrue(os.path.exists(tmpdir))
@@ -1546,6 +1831,25 @@ class ToolchainTest(EnhancedTestCase):
         # we may have created our own short tmpdir above, so make sure to clean things up...
         shutil.rmtree(orig_tmpdir)
 
+    def test_env_vars_external_module(self):
+        """Test env_vars_external_module function."""
+
+        res = env_vars_external_module('test', '1.2.3', {'prefix': '/software/test/1.2.3'})
+        expected = {'EBVERSIONTEST': '1.2.3', 'EBROOTTEST': '/software/test/1.2.3'}
+        self.assertEqual(res, expected)
+
+        res = env_vars_external_module('test-test', '1.2.3', {})
+        expected = {'EBVERSIONTESTMINTEST': '1.2.3'}
+        self.assertEqual(res, expected)
+
+        res = env_vars_external_module('test', None, {'prefix': '/software/test/1.2.3'})
+        expected = {'EBROOTTEST': '/software/test/1.2.3'}
+        self.assertEqual(res, expected)
+
+        res = env_vars_external_module('test', None, {})
+        expected = {}
+        self.assertEqual(res, expected)
+
 
 def suite():
     """ return all the tests"""
@@ -1553,4 +1857,5 @@ def suite():
 
 
 if __name__ == '__main__':
-    TextTestRunner(verbosity=1).run(suite())
+    res = TextTestRunner(verbosity=1).run(suite())
+    sys.exit(len(res.failures))
